@@ -62,12 +62,12 @@ program amr2
     use amr_module, only: cfl, cflv1, cflmax, evol
 
     use amr_module, only: checkpt_style, checkpt_interval, tchk, nchkpt
-    use amr_module, only: rstfile
+    use amr_module, only: rstfile, check_a
 
     use amr_module, only: max1d, maxvar, maxlv
 
     use amr_module, only: method, mthlim, use_fwaves, numgrids
-    use amr_module, only: nghost, mwaves, mcapa, auxtype
+    use amr_module, only: nghost, mwaves, mcapa, auxtype, dimensional_split
     use amr_module, only: tol, tolsp, flag_richardson, flag_gradient
 
     use amr_module, only: nghost, mthbc
@@ -79,30 +79,55 @@ program amr2
     use amr_module, only: output_aux_onlyonce, matlabu
 
     use amr_module, only: lfine, lentot, iregridcount, avenumgrids
-    use amr_module, only: tvoll, rvoll, rvol, mstart, possk, ibuff
+    use amr_module, only: tvoll, tvollCPU, rvoll, rvol, mstart, possk, ibuff
     use amr_module, only: timeRegridding,timeUpdating, timeValout
+    use amr_module, only: timeBound,timeStepgrid, timeFlagger,timeBufnst
+    use amr_module, only: timeBoundCPU,timeStepGridCPU,timeRegriddingCPU
+    use amr_module, only: timeValoutCPU,timeTick,timeTickCPU
     use amr_module, only: kcheck, iorder, lendim, lenmax
 
     use amr_module, only: dprint, eprint, edebug, gprint, nprint, pprint
     use amr_module, only: rprint, sprint, tprint, uprint
 
     use amr_module, only: t0, tstart_thisrun
+    use amr_module, only: tick_clock_start, tick_cpu_start
 
+
+    ! Data modules
     use regions_module, only: set_regions
     use gauges_module, only: set_gauges, num_gauges
+    use adjoint_module, only: read_adjoint_data
+
+#ifdef HDF5
+    use hdf5
+#endif
 
     implicit none
 
     ! Local variables
     integer :: i, iaux, mw, level
-    integer :: ndim, nvar, naux, mcapa1, mindim, dimensional_split
+    integer :: ndim, nvar, naux, mcapa1, mindim
     integer :: nstart, nsteps, nv1, nx, ny, lentotsave, num_gauge_SAVE
     integer :: omp_get_max_threads, maxthreads
     real(kind=8) :: time, ratmet, cut, dtinit, dt_max
     logical :: vtime, rest, output_t0    
 
     ! Timing variables
-    integer :: clock_start, clock_finish, clock_rate
+    integer(kind=8) :: clock_start, clock_finish, clock_rate, ttotal, count_max
+    real(kind=8) ::ttotalcpu
+    integer(kind=8) :: tick_clock_finish
+    integer, parameter :: timing_unit = 48
+    character(len=512) :: timing_line, timing_substr
+    character(len=*), parameter :: timing_base_name = "timing."
+    character(len=*), parameter :: timing_header_format =                      &
+                                                  "(' wall time (', i2,')," // &
+                                                  " CPU time (', i2,'), "   // &
+                                                  "cells updated (', i2,'),')"
+
+#ifdef HDF5
+    ! HDF5 Output
+    integer :: hdf_error
+#endif
 
     ! Common block variables
     real(kind=8) :: dxmin, dymin
@@ -196,6 +221,17 @@ program amr2
         read(inunit,*) (output_aux_components(i),i=1,naux)
         read(inunit,*) output_aux_onlyonce
     endif
+    if (output_format == 4) then
+#ifdef HDF5
+        ! Initialize HDF interface
+        call h5open_f(hdf_error)
+#else                    
+        print *, "ERROR:  HDF5 library is not available."
+        print *, "  Check the documentation as to how to include"
+        print *, "  the ability to output in HDF5 formats."
+        stop
+#endif
+    end if
     ! ==========================================================================
 
     ! ==========================================================================
@@ -208,7 +244,8 @@ program amr2
     read(inunit,*) nv1        ! steps_max
       
     if (output_style /= 3) then
-        nstop = nv1
+        !nstop = nv1
+        nstop = iinfinity   ! basically disabled this test
     endif
 
     read(inunit,*) vtime      ! dt_variable
@@ -223,11 +260,11 @@ program amr2
     read(inunit,*) method(3)  ! order_trans
 
     read(inunit,*) dimensional_split
-    if (dimensional_split > 0) then
-        print *, '*** ERROR ***  dimensional_split = ', dimensional_split
-        print *, ' dimensional splitting not supported in amrclaw'
-        stop
-    endif
+       if (dimensional_split > 1) then
+           print *, '*** ERROR ***  dimensional_split = ', dimensional_split
+           print *, ' Strang splitting not supported in amrclaw'
+           stop
+       endif
 
     read(inunit,*) method(4)   ! verbosity
     read(inunit,*) method(5)   ! src_split
@@ -289,12 +326,12 @@ program amr2
         ! Never checkpoint:
         checkpt_interval = iinfinity
 
-    else if (checkpt_style == 2) then
+    else if (abs(checkpt_style) == 2) then
         read(inunit,*) nchkpt
         allocate(tchk(nchkpt))
         read(inunit,*) (tchk(i), i=1,nchkpt)
 
-    else if (checkpt_style == 3) then
+    else if (abs(checkpt_style) == 3) then
         ! Checkpoint every checkpt_interval steps on coarse grid
         read(inunit,*) checkpt_interval
     endif
@@ -336,7 +373,7 @@ program amr2
         read(inunit,*) (auxtype(iaux), iaux=1,naux)
     endif
     read(inunit,*)
-              
+
     read(inunit,*) flag_richardson
     read(inunit,*) tol            ! for richardson
     read(inunit,*) flag_gradient
@@ -362,11 +399,6 @@ program amr2
     ! Finished with reading in parameters
     ! ==========================================================================
 
-    ! Read in region and gauge data
-    call set_regions('regions.data')
-    call set_gauges('gauges.data')
-
-
     ! Look for capacity function via auxtypes:
     mcapa = 0
     do iaux = 1, naux
@@ -404,7 +436,7 @@ program amr2
         print *, 'Error ***   need finer domain >', mindim, ' cells'
         stop
     endif
-    if (mcapa > naux) then     
+    if (mcapa > naux) then
         stop 'Error ***   mcapa > naux in input file'
     endif
 
@@ -435,6 +467,11 @@ program amr2
         matlabu   = 1
     endif
 
+    ! Boolean check_a tells which checkpoint file to use next if alternating
+    ! between only two files via check_twofiles.f, unused otherwise.
+    ! May be reset in call to restrt, otherwise default to using aaaaa file.
+    check_a = .true.   
+
     if (rest) then
 
         open(outunit, file=outfile, status='unknown', position='append', &
@@ -450,7 +487,25 @@ program amr2
         ! Call user routine to set up problem parameters:
         call setprob()
 
-    else
+        ! Non-user data files
+        call set_regions()
+        call set_gauges(rest, nvar, naux)
+        call read_adjoint_data()          ! Read adjoint solution
+
+    else        
+
+        ! Create new timing file
+        open(unit=timing_unit, file=timing_base_name//"csv",                &
+             form='formatted', status='unknown', action='write')
+        ! Construct header string
+        timing_line = 'output_time,total_wall_time,total_cpu_time,'
+        timing_substr = ""
+        do level=1, mxnest
+            write(timing_substr, timing_header_format) level, level, level
+            timing_line = trim(timing_line) // trim(timing_substr)
+        end do
+        write(timing_unit, "(a)") timing_line
+        close(timing_unit)
 
         open(outunit, file=outfile, status='unknown', form='formatted')
 
@@ -459,6 +514,11 @@ program amr2
         ! Call user routine to set up problem parameters:
         call setprob()
 
+        ! Non-user data files
+        call set_regions()
+        call set_gauges(rest, nvar, naux)
+        call read_adjoint_data()          ! Read adjoint solution
+
         cflmax = 0.d0   ! otherwise use previously heckpointed val
 
         lentot = 0
@@ -501,7 +561,6 @@ program amr2
         nstart = 0
     endif
 
-
     write(parmunit,*) ' '
     write(parmunit,*) '--------------------------------------------'
     write(parmunit,*) ' '
@@ -550,6 +609,9 @@ program amr2
     print *, 'Done reading data, starting computation ...  '
     print *, ' '
 
+    ! initialize timers before calling valout:
+    call system_clock(tick_clock_start, clock_rate)
+    call cpu_time(tick_cpu_start)
 
     call outtre (mstart,printout,nvar,naux)
     write(outunit,*) "  original total mass ..."
@@ -559,8 +621,6 @@ program amr2
     endif
     close(parmunit)
 
-    ! Timing
-    call system_clock(clock_start,clock_rate)
 
     ! --------------------------------------------------------
     !  Tick is the main routine which drives the computation:
@@ -569,38 +629,144 @@ program amr2
     call tick(nvar,cut,nstart,vtime,time,naux,t0,rest,dt_max)
     ! --------------------------------------------------------
 
-    call system_clock(clock_finish,clock_rate)
-    write(*,*) " "
-    write(outunit,*) " "
-    format_string = "('Total time to solution = ',1f16.8,' s, using ',i3,' threads')"
-    write(outunit,format_string) &
-            real(clock_finish - clock_start,kind=8) / real(clock_rate,kind=8), maxthreads
-    write(*,format_string) &
-            real(clock_finish - clock_start,kind=8) / real(clock_rate,kind=8), maxthreads
+    ! call system_clock to get clock_finish and count_max for debug output:
+    call system_clock(clock_finish,clock_rate,count_max)
 
-    do level = 1, mxnest            
-      format_string = "('Total advanc time on level ',i3,' = ',1f16.8,' s')"
-      write(outunit,format_string) level, &
-             real(tvoll(level),kind=8) / real(clock_rate,kind=8)
-      write(*,format_string) level, &
-             real(tvoll(level),kind=8) / real(clock_rate,kind=8)
+    !output timing data
+    open(timing_unit, file=timing_base_name//"txt", status='unknown',       &
+         form='formatted')
+    write(*,*)
+    write(timing_unit,*)
+    format_string="('============================== Timing Data ==============================')"
+    write(timing_unit,format_string)
+    write(*,format_string)
+    
+    write(*,*)
+    write(timing_unit,*)
+    
+    !Integration time
+    format_string="('Integration Time (stepgrid + BC + overhead)')"
+    write(timing_unit,format_string)
+    write(*,format_string)
+    
+    !Advanc time
+    format_string="('Level           Wall Time (seconds)    CPU Time (seconds)   Total Cell Updates')"
+    write(timing_unit,format_string)
+    write(*,format_string)
+
+    ! level counters are cumulative after restart, so initialize sums to zero 
+    ! even after restart to sum up time over all levels
+    ttotalcpu=0.d0
+    ttotal=0
+      
+    do level=1,mxnest
+        format_string="(i3,'           ',1f15.3,'        ',1f15.3,'    ', e17.3)"
+        write(timing_unit,format_string) level, &
+             real(tvoll(level),kind=8) / real(clock_rate,kind=8), tvollCPU(level), rvoll(level)
+        write(*,format_string) level, &
+             real(tvoll(level),kind=8) / real(clock_rate,kind=8), tvollCPU(level), rvoll(level)
+        ttotalcpu=ttotalcpu+tvollCPU(level)
+        ttotal=ttotal+tvoll(level)
     end do
-    write(*,*) " "
-    write(outunit,*)" "
+    
+    format_string="('total         ',1f15.3,'        ',1f15.3,'    ', e17.3)"
+    write(timing_unit,format_string) &
+             real(ttotal,kind=8) / real(clock_rate,kind=8), ttotalCPU, rvol
+    write(*,format_string) &
+             real(ttotal,kind=8) / real(clock_rate,kind=8), ttotalCPU, rvol
+    
+    write(*,*)
+    write(timing_unit,*)
+    
+    
+    format_string="('All levels:')"
+    write(*,format_string)
+    write(timing_unit,format_string)
+    
+    !stepgrid
+    format_string="('stepgrid      ',1f15.3,'        ',1f15.3,'    ',e17.3)"
+    write(timing_unit,format_string) &
+         real(timeStepgrid,kind=8) / real(clock_rate,kind=8), timeStepgridCPU
+    write(*,format_string) &
+         real(timeStepgrid,kind=8) / real(clock_rate,kind=8), timeStepgridCPU
+    
+    !bound
+    format_string="('BC/ghost cells',1f15.3,'        ',1f15.3)"
+    write(timing_unit,format_string) &
+         real(timeBound,kind=8) / real(clock_rate,kind=8), timeBoundCPU
+    write(*,format_string) &
+         real(timeBound,kind=8) / real(clock_rate,kind=8), timeBoundCPU
+    
+    !regridding time
+    format_string="('Regridding    ',1f15.3,'        ',1f15.3,'  ')"
+    write(timing_unit,format_string) &
+            real(timeRegridding,kind=8) / real(clock_rate,kind=8), timeRegriddingCPU
+    write(*,format_string) &
+            real(timeRegridding,kind=8) / real(clock_rate,kind=8), timeRegriddingCPU
+    
+    !output time
+    format_string="('Output (valout)',1f14.3,'        ',1f15.3,'  ')"
+    write(timing_unit,format_string) &
+            real(timeValout,kind=8) / real(clock_rate,kind=8), timeValoutCPU
+    write(*,format_string) &
+            real(timeValout,kind=8) / real(clock_rate,kind=8), timeValoutCPU
+    
+    write(*,*)
+    write(timing_unit,*)
+    
+    !Total Time
+    format_string="('Total time:   ',1f15.3,'        ',1f15.3,'  ')"
 
-    format_string = "('Total updating   time            ',1f16.8,' s')"
-    write(outunit,format_string)  real(timeUpdating,kind=8) / real(clock_rate,kind=8)
-    write(*,format_string) real(timeUpdating,kind=8) / real(clock_rate,kind=8)
+    write(*,format_string) real(timeTick,kind=8)/real(clock_rate,kind=8), &
+            timeTickCPU
+    write(timing_unit,format_string) real(timeTick,kind=8)/real(clock_rate,kind=8), &
+            timeTickCPU
 
-    format_string = "('Total valout     time            ',1f16.8,' s')"
-    write(outunit,format_string)  real(timeValout,kind=8) / real(clock_rate,kind=8)
-    write(*,format_string) real(timeValout,kind=8) / real(clock_rate,kind=8)
+    
+    format_string="('Using',i3,' thread(s)')"
+    write(timing_unit,format_string) maxthreads
+    write(*,format_string) maxthreads
+    
+    
+    write(*,*)
+    write(timing_unit,*)
+    
+    
+    write(*,"('Note: The CPU times are summed over all threads.')")
+    write(timing_unit,"('Note: The CPU times are summed over all threads.')")
+    write(*,"('      Total time includes more than the subroutines listed above')")
+    write(timing_unit,"('      Total time includes more than the subroutines listed above')")
+    if (rest) then
+      write(*,"('      Times for restart runs are cumulative')")
+      write(timing_unit,"('      Times for restart runs are cumulative')")
+    endif
 
-    format_string = "('Total regridding time            ',1f16.8,' s')"
-    write(outunit,format_string)  &
-             real(timeRegridding,kind=8) / real(clock_rate,kind=8)
-    write(*,format_string)  &
-             real(timeRegridding,kind=8) / real(clock_rate,kind=8)
+    write(*, "('Note: timings are also recorded for each output step')")
+    write(*, "('      in the file timing.csv.')")
+    write(timing_unit, "('Note: timings are also recorded for each output step')")
+    write(timing_unit, "('      in the file timing.csv.')")
+    
+    
+    write(*,*)
+    write(timing_unit,*)
+
+    ! output clock_rate etc. useful in debugging or if negative time reported
+
+    format_string="('clock_rate = ',i10, ' per second,  count_max = ',i23)"
+    !write(*,format_string) clock_rate, count_max
+    write(timing_unit,format_string) clock_rate, count_max
+
+    format_string="('clock_start = ',i20, ',  clock_finish = ',i20)"
+    !write(*,format_string) tick_clock_start, clock_finish
+    write(timing_unit,format_string) tick_clock_start, clock_finish
+
+    format_string="('=========================================================================')"
+    write(timing_unit,format_string)
+    write(*,format_string)
+    write(*,*)
+    write(timing_unit,*)
+    close(timing_unit)
+    !end of timing data
 
     ! Done with computation, cleanup:
     lentotsave = lentot
