@@ -9,10 +9,11 @@ c
 
        double precision valdub(nvar,midub,mjdub)
        double precision auxdub(naux,midub,mjdub)
+       double precision valbgc(nvar,mi2tot,mj2tot)
+       double precision auxbgc(naux,mi2tot,mj2tot)
        dimension fp(nvar,mi2tot,mj2tot),gp(nvar,mi2tot,mj2tot)
        dimension fm(nvar,mi2tot,mj2tot),gm(nvar,mi2tot,mj2tot)
-
-
+       
           hx  = hxposs(lcheck)
           hy  = hyposs(lcheck)
           hx2 = 2.d0*hx
@@ -41,6 +42,7 @@ c
               mx = midub - 4*nghost
               my = mjdub - 4*nghost
               auxdub = NEEDS_TO_BE_SET  ! signal that needs a val
+              
               call setaux(2*nghost,mx,my,xl,yb,hx,hy,
      &                    naux,auxdub)
               call auxcoarsen(auxdub,midub,mjdub,
@@ -48,7 +50,6 @@ c
           endif
 
 c         # fill it - use enlarged (before coarsening) aux arrays
-c          call bound(tpre,nvar,ng2,alloc(locdub),midub,mjdub,mptr,
           call bound(tpre,nvar,ng2,valdub,midub,mjdub,mptr,
      1               auxdub,naux)
 
