@@ -1,6 +1,26 @@
 c
 c -------------------------------------------------------------
 c
+!> Take a time step on a single grid **mptr** and overwrite solution array **q**. 
+!! A modified version of the clawpack routine step2 is used.
+!! 
+!! Return new solution **q** as well as fluxes in fm,fp and gm,gp.
+!! Patch has room for ghost cells (mbc of them) around the grid.
+!! Everything is the enlarged size (**mitot** by **mjtot**).
+!! 
+!! \param[in] mbc number of ghost cells  (= lwidth)
+!! \param[in] mptr grid number (for debugging)
+!! \param[in] xlow left edge of enlarged grid (including ghost cells).
+!! \param[in] ylow lower edge of enlarged grid (including ghost cells).
+!! \param[in] dt incoming time step
+!! \param[in] dx mesh size in x direction for this grid
+!! \param[in] dx mesh size in y direction for this grid
+!! \param[in,out] q solution array
+!! \param[out] dtnew  return suggested new time step for this grid's soln.
+!! \param[out] fm fluxes on the left side of each vertical edge
+!! \param[out] fp fluxes on the right side of each vertical edge
+!! \param[out] gm fluxes on the lower side of each horizontal edge
+!! \param[out] gp fluxes on the upper side of each horizontal edge
       subroutine stepgrid(q,fm,fp,gm,gp,mitot,mjtot,mbc,dt,dtnew,dx,dy,
      &                  nvar,xlow,ylow,time,mptr,maux,aux)
 c
@@ -35,7 +55,7 @@ c :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
       dimension fp(nvar,mitot,mjtot),gp(nvar,mitot,mjtot)
       dimension fm(nvar,mitot,mjtot),gm(nvar,mitot,mjtot)
       dimension aux(maux,mitot,mjtot)
-C 	  dimension work(mwork)
+c 	  dimension work(mwork)
 
       logical    debug,  dump
       data       debug/.false./,  dump/.false./
@@ -51,9 +71,9 @@ c     # needed there.
          write(outunit,*) "dumping grid ",mptr," at time ",time
          do i = 1, mitot
          do j = 1, mjtot
-            write(outunit,545) i,j,(q(ivar,i,j),ivar=1,nvar),
-     .                  (aux(ivar,i,j),ivar=1,maux)
- 545        format(2i4,4e15.7)
+            write(outunit,545) i,j,(q(ivar,i,j),ivar=1,nvar) 
+c    .                  ,(aux(ivar,i,j),ivar=1,maux)
+ 545        format(2i4,5e15.7)
          end do
          end do
       endif
@@ -83,7 +103,7 @@ C          write(*      ,*) 'CLAW2 ERROR... mwork must be increased to ',
 C      &               mwork0
 C          stop
 C       endif
-	  
+c  
 c
 c     # partition work array into pieces needed for local storage in 
 c     # step2 routine. Find starting index of each piece:
@@ -118,9 +138,9 @@ c
      &              fm,fp,gm,gp,rpn2,rpt2)
 c
 c
-        write(outunit,1001) mptr, node(nestlevel,mptr),cflgrid
- 1001   format(' Courant # of grid', i4,
-     &        ' on level', i3, ' is  ', e10.3)
+c       write(outunit,1001) mptr, node(nestlevel,mptr),cflgrid
+c1001   format(' Courant # of grid', i4,
+c    &        ' on level', i3, ' is  ', e10.3)
 c
 
 !$OMP  CRITICAL (cflm)
