@@ -64,11 +64,11 @@ program amr3
     use amr_module, only: cfl, cflv1, cflmax, evol
 
     use amr_module, only: checkpt_style, checkpt_interval, tchk, nchkpt
-    use amr_module, only: rstfile
+    use amr_module, only: rstfile, check_a
 
     use amr_module, only: max1d, maxvar, maxlv
 
-    use amr_module, only: method, mthlim, use_fwaves
+    use amr_module, only: method, mthlim, use_fwaves,dimensional_split
     use amr_module, only: nghost, mwaves, mcapa, auxtype
     use amr_module, only: tol, tolsp, flag_richardson, flag_gradient
 
@@ -81,7 +81,11 @@ program amr3
     use amr_module, only: output_aux_onlyonce, matlabu
 
     use amr_module, only: lfine, lentot, iregridcount, avenumgrids
-    use amr_module, only: tvoll, rvoll, rvol, mstart, possk, ibuff
+    use amr_module, only: tvoll,tvollCPU, rvoll, rvol, mstart, possk, ibuff
+    use amr_module, only: timeRegridding,timeRegriddingCPU
+    use amr_module, only: timeBound,timeStepgrid
+    use amr_module, only: timeBoundCPU,timeStepgridCPU
+    use amr_module, only: timeValout,timeValoutCPU
     use amr_module, only: kcheck, iorder, lendim, lenmax
 
     use amr_module, only: dprint, eprint, edebug, gprint, nprint, pprint
@@ -90,21 +94,30 @@ program amr3
     use amr_module, only: t0, tstart_thisrun
 
     !use regions_module, only: set_regions
-    !use gauges_module, only: set_gauges, num_gauges
+    use gauges_module, only: set_gauges, num_gauges
 
     implicit none
 
     ! Local variables
     integer :: i, iaux, mw, level
-    integer :: ndim, nvar, naux, mcapa1, mindim, dimensional_split
+    integer :: ndim, nvar, naux, mcapa1, mindim
     integer :: nstart, nsteps, nv1, nx, ny, nz, lentotsave
-    !integer :: nstart, nsteps, nv1, nx, ny, lentotsave, num_gauge_SAVE
+    !integer :: nstart, nsteps, nv1, nx, ny, lentotsave
+    integer :: num_gauge_SAVE
     integer :: omp_get_max_threads, maxthreads
     real(kind=8) :: time, ratmet, cut, dtinit, dt_max
     logical :: vtime, rest, output_t0    
 
     ! Timing variables
-    integer :: clock_start, clock_finish, clock_rate
+    integer :: clock_start, clock_finish, clock_rate, ttotal
+    real(kind=8) :: cpu_start, cpu_finish,ttotalcpu
+    integer, parameter :: timing_unit = 48
+    character(len=512) :: timing_line, timing_substr
+    character(len=*), parameter :: timing_base_name = "timing."
+    character(len=*), parameter :: timing_header_format =                      &
+                                                  "(' wall time (', i2,')," // &
+                                                  " CPU time (', i2,'), "   // &
+                                                  "cells updated (', i2,'),')"
 
     ! Common block variables
     real(kind=8) :: dxmin, dymin, dzmin
@@ -113,10 +126,10 @@ program amr3
 
     character(len=364) :: format_string
     character(len=*), parameter :: clawfile = 'claw.data'
-    character(len=*), parameter :: amrfile = 'amr.data'
-    character(len=*), parameter :: outfile = 'fort.amr'
+    character(len=*), parameter :: amrfile  = 'amr.data'
+    character(len=*), parameter :: outfile  = 'fort.amr'
     character(len=*), parameter :: dbugfile = 'fort.debug'
-    character(len=*), parameter :: matfile = 'fort.nplot'
+    character(len=*), parameter :: matfile  = 'fort.nplot'
     character(len=*), parameter :: parmfile = 'fort.parameters'
 
     ! Open parameter and debug files
@@ -210,7 +223,8 @@ program amr3
     read(inunit,*) nv1        ! steps_max
       
     if (output_style /= 3) then
-        nstop = nv1
+        !nstop = nv1
+        nstop = iinfinity ! basically disables this test
     endif
 
     read(inunit,*) vtime      ! dt_variable
@@ -225,9 +239,9 @@ program amr3
     read(inunit,*) method(3)  ! order_trans
 
     read(inunit,*) dimensional_split
-    if (dimensional_split > 0) then
+    if (dimensional_split > 1) then
         print *, '*** ERROR ***  dimensional_split = ', dimensional_split
-        print *, ' dimensional splitting not supported in amrclaw'
+        print *, ' Strang splitting not supported in amrclaw'
         stop
     endif
 
@@ -300,12 +314,12 @@ program amr3
         ! Never checkpoint:
         checkpt_interval = iinfinity
 
-    else if (checkpt_style == 2) then
+    else if (abs(checkpt_style) == 2) then
         read(inunit,*) nchkpt
         allocate(tchk(nchkpt))
         read(inunit,*) (tchk(i), i=1,nchkpt)
 
-    else if (checkpt_style == 3) then
+    else if (abs(checkpt_style) == 3) then
         ! Checkpoint every checkpt_interval steps on coarse grid
         read(inunit,*) checkpt_interval
     endif
@@ -375,10 +389,6 @@ program amr3
     ! Finished with reading in parameters
     ! ==========================================================================
 
-    ! Read in region and gauge data
-    !call set_regions('regions.data')
-    !call set_gauges('gauges.data')
-
 
     ! Look for capacity function via auxtypes:
     mcapa = 0
@@ -451,6 +461,11 @@ program amr3
         matlabu   = 1
     endif
 
+    ! Boolean check_a tells which checkpoint file to use next if alternating
+    ! between only two files via check_twofiles.f, unused otherwise.
+    ! May be reset in call to restrt, otherwise default to using aaaaa file.
+    check_a = .true.   
+
     if (rest) then
 
         open(outunit, file=outfile, status='unknown', position='append', &
@@ -466,7 +481,20 @@ program amr3
         ! Call user routine to set up problem parameters:
         call setprob()
 
+        call set_gauges(rest, nvar, naux)
     else
+        ! Write header out and continue
+        open(unit=timing_unit, file=timing_base_name//"csv",            &
+             form='formatted', status='unknown', action='write')
+        ! Construct header string
+        timing_line = 'output_time,total_wall_time,total_cpu_time,'
+        timing_substr = ""
+        do level=1, mxnest
+            write(timing_substr, timing_header_format) level, level, level
+            timing_line = trim(timing_line) // trim(timing_substr)
+        end do
+        write(timing_unit, "(a)") timing_line
+        close(timing_unit)
 
         open(outunit, file=outfile, status='unknown', form='formatted')
 
@@ -474,6 +502,7 @@ program amr3
 
         ! Call user routine to set up problem parameters:
         call setprob()
+        call set_gauges(rest, nvar, naux)
 
         cflmax = 0.d0   ! otherwise use previously heckpointed val
 
@@ -499,10 +528,10 @@ program amr3
         ! Hold off on gauges until grids are set. 
         ! The fake call to advance at the very first timestep 
         ! looks at the gauge array but it is not yet built
-        !num_gauge_SAVE = num_gauges
-        !num_gauges = 0
+        num_gauge_SAVE = num_gauges
+        num_gauges = 0
         call setgrd(nvar,cut,naux,dtinit,t0)
-        !num_gauges = num_gauge_SAVE
+        num_gauges = num_gauge_SAVE
 
 ! commented out to match 4-x version
 !!$        if (possk(1) .gt. dtinit*cflv1/cfl .and. vtime) then
@@ -584,6 +613,7 @@ program amr3
 
     ! Timing
     call system_clock(clock_start,clock_rate)
+    call cpu_time(cpu_start)
 
     ! --------------------------------------------------------
     !  Tick is the main routine which drives the computation:
@@ -592,19 +622,135 @@ program amr3
     ! --------------------------------------------------------
 
     call system_clock(clock_finish,clock_rate)
-    format_string = "('Total time to solution = ',1f16.8,' s')"
-    write(outunit,format_string) &
-            real(clock_finish - clock_start,kind=8) / real(clock_rate,kind=8)
-    write(*,format_string) &
-            real(clock_finish - clock_start,kind=8) / real(clock_rate,kind=8)
-
-    do level = 1, mxnest            
-      format_string = "('Total advanc time on level ',i3,' = ',1f16.8,' s')"
-      write(outunit,format_string) level, &
-             real(tvoll(level),kind=8) / real(clock_rate,kind=8)
-      write(*,format_string) level, &
-             real(tvoll(level),kind=8) / real(clock_rate,kind=8)
+    call cpu_time(cpu_finish)
+    
+    !output timing data
+    open(timing_unit, file=timing_base_name//"txt", status='unknown',        &
+         form='formatted')
+    write(*,*)
+    write(timing_unit,*)
+    format_string="('============================== Timing Data ==============================')"
+    write(timing_unit,format_string)
+    write(*,format_string)
+    
+    write(*,*)
+    write(timing_unit,*)
+    
+    !Integration time
+    format_string="('Integration Time (stepgrid + BC + overhead)')"
+    write(timing_unit,format_string)
+    write(*,format_string)
+    
+    !Advanc time
+    format_string="('Level           Wall Time (seconds)    CPU Time (seconds)   Total Cell Updates')"
+    write(timing_unit,format_string)
+    write(*,format_string)
+    ttotalcpu=0.d0
+    ttotal=0
+    do level=1,mxnest
+        format_string="(i3,'           ',1f15.3,'        ',1f15.3,'    ', e17.3)"
+        write(timing_unit,format_string) level, &
+             real(tvoll(level),kind=8) / real(clock_rate,kind=8), tvollCPU(level), rvoll(level)
+        write(*,format_string) level, &
+             real(tvoll(level),kind=8) / real(clock_rate,kind=8), tvollCPU(level), rvoll(level)
+        ttotalcpu=ttotalcpu+tvollCPU(level)
+        ttotal=ttotal+tvoll(level)
     end do
+    
+    format_string="('total         ',1f15.3,'        ',1f15.3,'    ', e17.3)"
+    write(timing_unit,format_string) &
+             real(ttotal,kind=8) / real(clock_rate,kind=8), ttotalCPU, rvol
+    write(*,format_string) &
+             real(ttotal,kind=8) / real(clock_rate,kind=8), ttotalCPU, rvol
+    
+    write(*,*)
+    write(timing_unit,*)
+    
+    
+    format_string="('All levels:')"
+    write(*,format_string)
+    write(timing_unit,format_string)
+    
+    
+    
+    !stepgrid
+    format_string="('stepgrid      ',1f15.3,'        ',1f15.3,'    ',e17.3)"
+    write(timing_unit,format_string) &
+         real(timeStepgrid,kind=8) / real(clock_rate,kind=8), timeStepgridCPU
+    write(*,format_string) &
+         real(timeStepgrid,kind=8) / real(clock_rate,kind=8), timeStepgridCPU
+    
+    !bound
+    format_string="('BC/ghost cells',1f15.3,'        ',1f15.3)"
+    write(timing_unit,format_string) &
+         real(timeBound,kind=8) / real(clock_rate,kind=8), timeBoundCPU
+    write(*,format_string) &
+         real(timeBound,kind=8) / real(clock_rate,kind=8), timeBoundCPU
+    
+    !regridding time
+    format_string="('Regridding    ',1f15.3,'        ',1f15.3,'  ')"
+    write(timing_unit,format_string) &
+            real(timeRegridding,kind=8) / real(clock_rate,kind=8), timeRegriddingCPU
+    write(*,format_string) &
+            real(timeRegridding,kind=8) / real(clock_rate,kind=8), timeRegriddingCPU
+    
+    !output time
+    format_string="('Output (valout)',1f14.3,'        ',1f15.3,'  ')"
+    write(timing_unit,format_string) &
+            real(timeValout,kind=8) / real(clock_rate,kind=8), timeValoutCPU
+    write(*,format_string) &
+            real(timeValout,kind=8) / real(clock_rate,kind=8), timeValoutCPU
+    
+    write(*,*)
+    write(timing_unit,*)
+    
+    !Total Time
+    format_string="('Total time:   ',1f15.3,'        ',1f15.3,'  ')"
+    write(timing_unit,format_string) &
+            real(clock_finish - clock_start,kind=8) / real(clock_rate,kind=8), &
+            cpu_finish-cpu_start
+    write(*,format_string) &
+            real(clock_finish - clock_start,kind=8) / real(clock_rate,kind=8), &
+            cpu_finish-cpu_start
+    
+    format_string="('Using',i3,' thread(s)')"
+    write(timing_unit,format_string) maxthreads
+    write(*,format_string) maxthreads
+    
+    
+    write(*,*)
+    write(timing_unit,*)
+    
+    
+    write(*,"('Note: The CPU times are summed over all threads.')")
+    write(timing_unit,"('Note: The CPU times are summed over all threads.')")
+    write(*,"('      Total time includes more than the subroutines listed above')")
+    write(timing_unit,"('      Total time includes more than the subroutines listed above')")
+    
+    
+    !end of timing data
+    write(*,*)
+    write(timing_unit,*)
+    format_string="('=========================================================================')"
+    write(timing_unit,format_string)
+    write(*,format_string)
+    write(*,*)
+    write(timing_unit,*)
+    close(timing_unit)
+    
+    !format_string = "('Total time to solution = ',1f16.8,' s')"
+    !write(outunit,format_string) &
+    !        real(clock_finish - clock_start,kind=8) / real(clock_rate,kind=8)
+    !write(*,format_string) &
+    !        real(clock_finish - clock_start,kind=8) / real(clock_rate,kind=8)
+
+    !do level = 1, mxnest            
+    !  format_string = "('Total advanc time on level ',i3,' = ',1f16.8,' s')"
+    !  write(outunit,format_string) level, &
+    !         real(tvoll(level),kind=8) / real(clock_rate,kind=8)
+    !  write(*,format_string) level, &
+    !         real(tvoll(level),kind=8) / real(clock_rate,kind=8)
+    !end do
 
     ! Done with computation, cleanup:
     lentotsave = lentot
