@@ -8,7 +8,7 @@ module amr_module
     ! :::::   data structure info.
     ! ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
     integer, parameter :: rsize = 7
-    integer, parameter :: nsize = 19
+    integer, parameter :: nsize = 21
 
     !  :::::::   integer part of node descriptor
     integer, parameter :: levelptr  = 1
@@ -30,6 +30,8 @@ module amr_module
     integer, parameter :: numflags  = 17
     integer, parameter :: domflags_base  = 18
     integer, parameter :: domflags2  = 19
+    integer, parameter :: bndListSt  = 20
+    integer, parameter :: bndListNum = 21
 
     ! :::::::  real part of node descriptor
     integer, parameter :: cornxlo  = 1
@@ -45,6 +47,8 @@ module amr_module
     integer, parameter :: null = 0
     integer, parameter :: nil  = 0
 
+    integer, parameter :: gridNbor = 1  ! use first col, 2nd col is nextfee - the link
+
     ! :::::::  for flagging points   
     real(kind=8), parameter :: goodpt = 0.0
     real(kind=8), parameter :: badpt  = 2.0
@@ -55,26 +59,48 @@ module amr_module
     integer, parameter :: iplane = 1
     integer, parameter :: jplane = 2
     integer, parameter :: kplane = 3
-    integer, parameter :: maxgr = 5000
+    !! integer, parameter :: maxgr = 5000 ! No longer fixed
     integer, parameter :: maxlv = 10
     integer, parameter :: maxcl = 5000
 
     ! The max1d parameter should be changed if using OpenMP grid based 
-    ! looping, usually set to max1d = 60
-    integer, parameter :: max1d = 60 
+    ! looping, suggest setting max1d = 32 for 3d 
+    integer, parameter :: max1d = 32 
+    !integer, parameter :: max1d = 28 
 
     integer, parameter :: maxvar = 10
     integer, parameter :: maxaux = 20
     integer, parameter :: maxwave = 10
     integer, parameter :: maxout = 50  !until change amr to f90 and allocate
 
-    real(kind=8) hxposs(maxlv),hyposs(maxlv),hzposs(maxlv), &
-                 possk(maxlv),rnode(rsize, maxgr) 
+    ! put linked list of grids into array and save.
+    ! order is coarsest level to finest. is redone after regridding
+    ! and on restarting.  note use of sentinel in listStart
 
+    !! CHANGED mjb 6/15/18 to make maxgr a variable
+    !!integer :: listOfGrids(maxgr),listStart(0:maxlv+1)
+    integer :: listStart(0:maxlv+1)
+    !integer, parameter :: bndListSize = 8*maxgr
+    !integer :: bndList(bndListSize,2) ! guess size, average # nbors 6? manage as linked list
+    integer :: bndListSize
+    integer,allocatable, dimension(:,:) :: bndList  ! new way is allocatable 
+
+
+    !real(kind=8) hxposs(maxlv),hyposs(maxlv),hzposs(maxlv), &
+    !             possk(maxlv),rnode(rsize, maxgr) 
+    real(kind=8) hxposs(maxlv),hyposs(maxlv),hzposs(maxlv),possk(maxlv) 
+
+    ! start of dynamic allocation for maxgr and associated arrays
+    integer maxgr
+    real(kind=8), allocatable, dimension(:,:) :: rnode    
+    ! new way, use allocatable, not pointer
+    integer, allocatable, dimension(:,:) :: node
+    integer, allocatable, dimension(:) :: listOfGrids
 
 
     real(kind=8) tol, tolsp
-    integer ibuff,  mstart, ndfree, lfine, node(nsize, maxgr), &
+    !integer ibuff, mstart,ndfree,ndfree_bnd,lfine,node(nsize, maxgr), &
+    integer ibuff, mstart,ndfree,ndfree_bnd,lfine, &
             icheck(maxlv),lstart(maxlv),newstl(maxlv), &
             listsp(maxlv),intratx(maxlv),intraty(maxlv), &
             intratz(maxlv),kratio(maxlv), &
@@ -93,8 +119,9 @@ module amr_module
     ! common  /calloc/   alloc(memsize)
 
     ! Dynamic memory: 
-    real(kind=8), allocatable, target, dimension(:) :: storage
-    real(kind=8), pointer, dimension(:) :: alloc
+    !real(kind=8), allocatable, target, dimension(:) :: storage
+    !real(kind=8), pointer, dimension(:) :: alloc
+    real(kind=8), allocatable, dimension(:) :: alloc  ! new way, use alloctable, not pointer
     integer memsize
        
     ! ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\
@@ -116,12 +143,18 @@ module amr_module
     real(kind=8)  rvoll(maxlv),evol,rvol,avenumgrids(maxlv)
     integer iregridcount(maxlv), tvoll(maxlv)
     integer lentot,lenmax,lendim
+    integer timeRegridding, timeValout
+    integer timeBound, timeStepgrid
+    integer :: timeTick, tick_clock_start
+    real(kind=8) tvollCPU(maxlv)
+    real(kind=8) timeBoundCPU,timeStepgridCPU,timeRegriddingCPU
+    real(kind=8) timeValoutCPU
 
     ! ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
     ! :::::  method parameters
     ! ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
     character(len=10), allocatable :: auxtype(:)
-    integer  method(7), mwaves, mcapa
+    integer  method(7), mwaves, mcapa, dimensional_split
     integer, allocatable :: mthlim(:)
     real(kind=8) cfl,cflmax,cflv1,cfl_level
 
@@ -153,6 +186,8 @@ module amr_module
 
     integer :: matlabu
 
+    !  USE UNITS NUMBERS < 89.
+    ! 89 and + numthreads taken by gauge output
     integer, parameter :: parmunit = 12
     integer, parameter :: chkunit = 10
     integer, parameter :: inunit  = 5
@@ -178,5 +213,6 @@ module amr_module
 
     ! Restart file name:
     character(len=200) :: rstfile
+    logical :: check_a
 
 end module amr_module
