@@ -43,13 +43,17 @@
 #---------------------------------------------------------------------------
 
 
+from __future__ import absolute_import
+from __future__ import print_function
 import sys,os,glob
-import string,re
+import re
 import time
+from six.moves import range
+from six.moves import input
 try:
     from optparse import OptionParser
 except:
-    print 'You must use a more recent version of Python'
+    print('You must use a more recent version of Python')
     sys.exit(1)
 
 # parse command line arguments:
@@ -89,13 +93,13 @@ eagle = options.eagle
 try:
     clawdir = os.getenv('CLAW')  # assumes environment variable set properly
 except:
-    print "CLAW environment variable not set"
-    print "You need to run setenv.py before setup.py"
+    print("CLAW environment variable not set")
+    print("You need to run setenv.py before setup.py")
     sys.exit(1)
 
 if clawdir == None:
-    print "CLAW environment variable not set"
-    print "You need to run setenv.py before setup.py"
+    print("CLAW environment variable not set")
+    print("You need to run setenv.py before setup.py")
     sys.exit(1)
 
 
@@ -105,9 +109,7 @@ if clawdir == None:
 
 #clawaddr = 'http://localhost:50005'
 
-clawaddr = 'http://clawpack.github.io/'
-# This doesn't work...  things are not found in the same places they were 
-# in Clawpack 4.x, so the favicon, logo, css files will not be found.
+clawaddr = 'http://www.clawpack.org/'
 
 
 # Set comment characters for different programming languages:
@@ -144,10 +146,10 @@ for infilename in infiles:
     if infilename == 'Makefile':
         ext = 'Makefile'  # special case
     
-    if not commentchar.has_key(ext):
-        print "  "
-        print "  Warning: Unrecognized extension, will proceed"
-        print "           with no replacement of comment characters"
+    if ext not in commentchar:
+        print("  ")
+        print("  Warning: Unrecognized extension, will proceed")
+        print("           with no replacement of comment characters")
         commentchar[ext] = None 
     
     
@@ -157,7 +159,7 @@ for infilename in infiles:
     try:
         ifile = open(infilename,'r')
     except:
-        print "File not found:", infilename
+        print("File not found:", infilename)
         sys.exit(1)
 
     # Search for [use: ...] statements in file:
@@ -188,9 +190,9 @@ for infilename in infiles:
 
     if (glob.glob(outfilename) != []) & (not forced):
         sys.stdout.write('  OK to overwrite %s?  '  %  outfilename)
-        answer = raw_input()
+        answer = input()
         if answer not in ['y','Y','yes']:
-            print '  Aborting!'
+            print('  Aborting!')
             sys.exit(1)
     
     ofile = open(outfilename,'w')
@@ -200,7 +202,7 @@ for infilename in infiles:
     #--------------------------
     
     if verbose:
-        print '  Converting ', infilename, ' to ', outfilename
+        print('  Converting ', infilename, ' to ', outfilename)
 
     
     ofile.write("""<!-- **** DO NOT EDIT THIS FILE *** 
@@ -258,7 +260,7 @@ for infilename in infiles:
         ofile.write('&nbsp;<font size=6> %s </font> </td>\n' % outfilename)
     
         ofile.write("""
-            <td><a href="http://clawpack.github.io/doc/index.html"><img
+            <td><a href="http://www.clawpack.org/index.html"><img
             src="%s/clawlogo.jpg"
             width=100 alt="CLAWPACK"></a>""" % clawaddr)
 
@@ -329,18 +331,18 @@ for infilename in infiles:
     
         lineno += 1
     
-        if string.count(line,"begin_html"):
+        if 'begin_html' in line:
             regexp = re.compile(r"\[color:(?P<color>[^\]]*)\]")
             result = regexp.search(line)
-	    if result:
-	        font_color = result.group('color')
-	    else:
-	        font_color = default_color
+            if result:
+                font_color = result.group('color')
+            else:
+                font_color = default_color
     
             if insidehtml:
-                print '  Error at line ', lineno, '\n'
-                print '  Unexpected begin_html  when already in html mode\n'
-                print '  Missing end_html? \n'
+                print('  Error at line ', lineno, '\n')
+                print('  Unexpected begin_html  when already in html mode\n')
+                print('  Missing end_html? \n')
                 sys.exit(1)
     
             # switch out of pre-formatted mode and create table to indent
@@ -360,8 +362,8 @@ for infilename in infiles:
                     ofile.write(' ')
                 ofile.write('</tt></pre>')
             else:
-                print '  Strange error in clawcode2html - should not be here'
-                print '     at line number ', lineno
+                print('  Strange error in clawcode2html - should not be here')
+                print('     at line number ', lineno)
               
     
             ofile.write('\n</td>\n')
@@ -373,7 +375,7 @@ for infilename in infiles:
                 ofile.write('<font size="-1" color=%s>\n'  % font_color)
             insidehtml = 1;
     
-        elif string.count(line,"end_html"):
+        elif 'end_html' in line:
             # switch back to pre-formatted environment
             ofile.write('</font></td></tr></table>\n')
             ofile.write('<pre> \n')
@@ -383,14 +385,14 @@ for infilename in infiles:
             if insidehtml:
     
                 # replace blank line in html comment by new paragraph <p>:
-                blankline = (string.split(line) == [])
+                blankline = (line.strip == '')
                 if not blankline:
-                   firstchar = string.split(line)[0][0]
+                   firstchar = line.split()[0][0]
                    if ((ext in ['.f','.f95']) & (firstchar not in firstfort)): 
                        if firstchar not in commentchar[ext]:
-                           print '  Error... in line ', lineno,'\n' \
+                           print('  Error... in line ', lineno,'\n' \
                                  '  In html but not in a comment.'\
-                                 '   Forgotten "end_html" ?'
+                                 '   Forgotten "end_html" ?')
                            sys.exit(1)
     
                 # fortran comments may have comment symbol in column 1
@@ -405,10 +407,10 @@ for infilename in infiles:
                 # replace any comment character by ' ' 
                 if commentchar[ext]:
                     for char in commentchar[ext]:
-                        line = string.replace(line,char,' ')  
+                        line = line.replace(char,' ')  
     
     
-                blankline = (string.split(line) == [])
+                blankline = (line.strip == '')
                 if blankline:
                    line = ('<p>\n')
 
@@ -416,21 +418,21 @@ for infilename in infiles:
                 # Allow wiki formatting of links:
                 # -------------------------------
 
-		# Replace [name: placemark] by <a name="placemark">
+                # Replace [name: placemark] by <a name="placemark">
                 # (to jump to a different spot in the same html file)
                 regexp = re.compile(r"\[name:[ ]*(?P<placemark>[^ ^\]]*)\]")
                 result = regexp.search(line)
                 while result:
                     placemark = result.group('placemark')
                     oldpat = result.group()
-		    newpat = '<a name="%s">' % placemark
+                    newpat = '<a name="%s">' % placemark
                     line = line.replace(oldpat,newpat)
                     result = regexp.search(line)
 
                 # Replace links of the form [code: target]
                 # by html links to both target and target.html.
-		# Also allows [code: target#placemark] with links to target 
-		# and target.html#placemark.
+                # Also allows [code: target#placemark] with links to target 
+                # and target.html#placemark.
 
                 regexp = re.compile(r"\[code:[ ]*(?P<target>[^ ^\]^#]*)([#]?)" + \
                                     r"(?P<placemark>[^\]]*)\]")
@@ -439,17 +441,17 @@ for infilename in infiles:
                     targetname = result.group('target')
                     placemark = result.group('placemark')
                     oldpat = result.group()
-		    if placemark:
+                    if placemark:
                         newpat = '<a href="%s">%s</a>' \
-			             % (targetname,targetname) + \
+                                     % (targetname,targetname) + \
                                  '&nbsp;<a href="%s.html#%s">[.html]</a>' \
-			             % (targetname,placemark) 
-		    else:
+                                     % (targetname,placemark) 
+                    else:
                         oldpat = result.group()
                         newpat = '<a href="%s">%s</a>' \
-			             % (targetname,targetname) + \
+                                     % (targetname,targetname) + \
                                  '&nbsp;<a href="%s.html">[.html]</a>' \
-			             % targetname 
+                                     % targetname 
                     line = line.replace(oldpat,newpat)
                     result = regexp.search(line)
 
@@ -505,11 +507,11 @@ for infilename in infiles:
                     result = regexp.search(line)
 
 
-		# special things for CLAWPACK:
+                # special things for CLAWPACK:
 
                 # replace links of the form [clawcode:clawpack/1d/lib/step1.f]
                 # for example by links relative to clawaddr, 
-		# along with a link to the .html version.
+                # along with a link to the .html version.
                 regexp = re.compile(r"\[clawcode:[ ]*(?P<target>[^ ^\]^#]*)([#]?)" + \
                                     r"(?P<placemark>[^\]]*)\]")
                 result = regexp.search(line)
@@ -517,23 +519,23 @@ for infilename in infiles:
                     targetname = result.group('target').lstrip()
                     placemark = result.group('placemark')
                     oldpat = result.group()
-		    if placemark:
+                    if placemark:
                         newpat = '<a href="%s/%s">claw/%s</a>' \
-			             % (clawaddr,targetname,targetname) + \
+                                     % (clawaddr,targetname,targetname) + \
                                  '&nbsp;<a href="%s/%s.html#%s">[.html]</a>' \
-			             % (clawaddr,targetname,placemark) 
-		    else:
+                                     % (clawaddr,targetname,placemark) 
+                    else:
                         newpat = '<a href="%s/%s">%s</a>' \
-			             % (clawaddr,targetname,targetname) + \
+                                     % (clawaddr,targetname,targetname) + \
                                  '&nbsp;<a href="%s/%s.html">[.html]</a>' \
-			             % (clawaddr,targetname) 
+                                     % (clawaddr,targetname) 
                     line = line.replace(oldpat,newpat)
                     result = regexp.search(line)
 
 
                 # replace links of the form [claw:clawpack/1d/lib]
                 # for example by links relative to clawaddr,
-		# with no .html version.
+                # with no .html version.
                 regexp = re.compile(r"\[claw:[ ]?(?P<target>[^ ^\]]*)" + \
                                     r"([ ]*)(?P<text>[^\]]*)\]")
                 result = regexp.search(line)
@@ -548,34 +550,34 @@ for infilename in infiles:
                     line = line.replace(oldpat,newpat)
                     result = regexp.search(line)
 
-		# place text surrounded by triple braces with 
-		# pre environment with background color:
-		newpat = '<pre class="clawcode">'
+                # place text surrounded by triple braces with 
+                # pre environment with background color:
+                newpat = '<pre class="clawcode">'
                 line = line.replace('{{{',newpat)
                 line = line.replace('}}}','</pre>')
 
           
-	    else:
+            else:
                 # not insidehtml - make regular comments default_color.
                 # Determine if this line contains a comment and if so,
                 # what column the comment starts in:
 
-		startcomment = 1000
-	        if (ext == '.f') & (line[0] in firstfort):
-	            startcomment = 0
-	        elif (ext == '.f95') & (line[0] in firstfort95):
-	            startcomment = 0
-		else:
-		    if commentchar[ext]:
+                startcomment = 1000
+                if (ext == '.f') & (line[0] in firstfort):
+                    startcomment = 0
+                elif (ext == '.f95') & (line[0] in firstfort95):
+                    startcomment = 0
+                else:
+                    if commentchar[ext]:
                         for c in commentchar[ext]:
-		            commentcol = string.find(line,c)
-		            if (commentcol>-1)&(commentcol<startcomment):
+                            commentcol = line.find(c)
+                            if (commentcol>-1)&(commentcol<startcomment):
                                 startcomment = commentcol
 
-		if startcomment<1000:
-		    line = line[0:startcomment] + \
-		           '<font color="%s">'  % default_color + \
-		           line[startcomment:-1] + '</font>\n'
+                if startcomment<1000:
+                    line = line[0:startcomment] + \
+                           '<font color="%s">'  % default_color + \
+                           line[startcomment:-1] + '</font>\n'
 
 
             # output the (possibly modified) line to the output file:
