@@ -11,11 +11,146 @@ Changes in 5.0:
 
 """
 
+from __future__ import absolute_import
+from __future__ import print_function
 import os
+import sys
+import inspect
+import tarfile
+import zipfile
 import string
+import six
+from six.moves import range
+from six.moves import input
+
+try:
+    from urllib.request import urlopen
+except ImportError:
+    # Fall back to Python 2's urllib2
+    from urllib2 import urlopen
 
 import numpy as np
 
+# ======================
+#  Remote file handling
+# ======================
+def strip_archive_extensions(path, 
+                             extensions=["tar", "tgz", "bz2", "gz", "zip"]):
+    r"""
+    Strip off archive extensions defined in *extensions* list.
+
+    Return stripped path calling this function recursively until all splitext
+    does not provide an extension in the *extensions* list.  Note that zip
+    files store the names of the contained files in the archive and the 
+    returned path will more than likely not have the appropriate file suffix.
+
+    """
+
+    if os.path.splitext(path)[-1][1:] in extensions:
+        return strip_archive_extensions(os.path.splitext(path)[0])
+    else:
+        return path
+
+
+def get_remote_file(url, output_dir=None, file_name=None, force=False,  
+                         verbose=False, ask_user=False, unpack=True):
+    r"""Fetch file located at *url* and store at *output_dir*.
+
+    :Input:
+    
+     - *url* (path) - URL to file to be downloaded.
+     - *output_dir* (path) - Directory that the remote file will be downloaded
+       to.  Defaults to the GeoClaw sratch directory defined by 
+       *os.path.join(os.environ['CLAW'], 'geoclaw', 'scratch')*.
+     - *file_name* (string) - Name of local file.  This defaults to the name of
+       the remote file.
+     - *force* (bool) - Force downloading of remote file regardless of whether
+       it exists locally or not.  Default is *False*
+     - *verbose* (bool) - Print out status information.  Default is *False*
+     - *ask_user* (bool) - Whether to ask the user if it is ok to download the
+       file before proceeding.  Default is *False*
+
+    :Raises:
+     
+    Exceptions are raised from the *urllib* module having to do with errors
+    fetching the remote file.  Please see its documentation for more details of
+    the exceptions that can be raised.
+
+    :Notes:
+
+    If fetching the file fails *urllib* does not always warn you so if errors
+    occur it may be wise to check to make sure the downloaded file actually has
+    content in it.
+
+    returns *unarchived_output_path*
+    """
+
+    if output_dir is None:
+        output_dir = os.path.join(os.environ['CLAW'], 'geoclaw', 'scratch')
+
+    if file_name is None:
+        file_name = os.path.basename(url)
+        
+    output_path = os.path.join(output_dir, file_name)
+    unarchived_output_path = strip_archive_extensions(output_path)
+
+    if not os.path.exists(unarchived_output_path) or force:
+
+        if ask_user:
+            ans = input("  Ok to download topo file and save as %s?  \n"
+                            % unarchived_output_path,
+                            "     Type y[es], n[o].")
+            if ans.lower() in ['y', 'yes']:
+                if verbose:
+                    print("*** Aborting download.")
+                return None
+            
+        # if not os.path.exists(output_path):
+        # Fetch remote file, will raise a variety of exceptions depending on
+        # the retrieval problem if it happens
+        if verbose:
+            print("Downloading %s to %s..." % (url, output_path))
+        with open(output_path, "wb") as output_file:
+            remote_file = urlopen(url)
+            output_file.write(remote_file.read())
+        if verbose:
+            print("Done downloading.")
+        # elif verbose:
+        #     print("File already exists, not downloading")
+
+        if tarfile.is_tarfile(output_path) and unpack:
+            if verbose:
+                print("Un-archiving %s to %s..." % (output_path, 
+                                                    unarchived_output_path))
+            with tarfile.open(output_path, mode="r:*") as tar_file:
+                tar_file.extractall(path=output_dir)
+            if verbose:
+                print("Done un-archiving.")
+        elif zipfile.is_zipfile(output_path) and unpack:
+            if verbose:
+                print("Un-archiving %s to %s..." % (output_path, 
+                                                    unarchived_output_path))
+            with zipfile.ZipFile(output_path, mode="r") as zip_file:
+                zip_file.extractall(path=output_dir)
+                # Add file suffix
+                extension = os.path.splitext(zip_file.namelist()[0])[-1]
+                unarchived_output_path = "".join((unarchived_output_path,
+                                                  extension))
+            if verbose:
+                print("Done un-archiving.")
+
+    else:
+        if verbose:
+            print("Skipping %s " % url)
+            print("  because file already exists: %s" % output_path)
+
+    if unpack:
+        return unarchived_output_path
+    else:
+        return output_path
+
+
+
 # ==============================================================================
 #  Base data class for Clawpack data objects
 class ClawData(object):
@@ -55,9 +190,9 @@ class ClawData(object):
         """
 
         if (not name in self._attributes) and (name[0] != '_'):
-            print "*** Unrecognized attribute: ",name
-            print "*** Perhaps a typo?"
-            print "*** Add new attributes using add_attribute method"
+            print("*** Unrecognized attribute: ",name)
+            print("*** Perhaps a typo?")
+            print("*** Add new attributes using add_attribute method")
             raise AttributeError("Unrecognized attribute: %s" % name)
         
         # attribute exists, ok to set:
@@ -67,7 +202,7 @@ class ClawData(object):
     def __str__(self):
         r"""Returns string representation of this object"""
         output = "%s%s\n" % ("Name".ljust(25),"Value".ljust(12))
-        for (k,v) in self.iteritems():
+        for (k,v) in six.iteritems(self):
             output += "%s%s\n" % (str(k).ljust(25),str(v).ljust(12))
         return output
 
@@ -152,7 +287,7 @@ class ClawData(object):
          - (file) - file object
         """
 
-        source = string.ljust(datasource,25)
+        source = datasource.ljust(25)
         self._out_file = open(name, 'w')
         self._out_file.write('########################################################\n')
         self._out_file.write('### DO NOT EDIT THIS FILE:  GENERATED AUTOMATICALLY ####\n')
@@ -167,7 +302,7 @@ class ClawData(object):
         self._out_file = None
 
 
-    def write(self,out_file,data_source='setrun.py'):
+    def write(self, out_file, data_source='setrun.py'):
         r"""Write out all data files in this ClawData object"""
 
         # Open data file
@@ -220,8 +355,8 @@ class ClawData(object):
                     string_value = 'F'
             else:
                 string_value = repr(value)
-            padded_value = string.ljust(string_value, 20)
-            padded_name = string.ljust(alt_name,20)
+            padded_value = string_value.ljust(20)
+            padded_name = alt_name.ljust(20)
             if description != '':
                 self._out_file.write('%s =: %s # %s \n' % 
                                         (padded_value, padded_name, description))
@@ -325,10 +460,10 @@ class ClawRunData(ClawData):
             import clawpack.amrclaw.data as amrclaw
 
             self.xclawcmd = 'xamr'
-            self.add_data(ClawInputData(num_dim),'clawdata')
             self.add_data(amrclaw.AmrclawInputData(self.clawdata),'amrdata')
             self.add_data(amrclaw.RegionData(num_dim=num_dim),'regiondata')
             self.add_data(amrclaw.GaugeData(num_dim=num_dim),'gaugedata')
+            self.add_data(amrclaw.AdjointData(num_dim=num_dim),'adjointdata')
 
         elif pkg.lower() in ['geoclaw']:
 
@@ -341,6 +476,7 @@ class ClawRunData(ClawData):
             self.add_data(amrclaw.AmrclawInputData(self.clawdata),'amrdata')
             self.add_data(amrclaw.RegionData(num_dim=num_dim),'regiondata')
             self.add_data(amrclaw.GaugeData(num_dim=num_dim),'gaugedata')
+            self.add_data(amrclaw.AdjointData(num_dim=num_dim),'adjointdata')
             self.add_data(geoclaw.GeoClawData(),('geo_data'))
             self.add_data(geoclaw.TopographyData(),'topo_data')
             self.add_data(geoclaw.DTopoData(),'dtopo_data')
@@ -348,13 +484,9 @@ class ClawRunData(ClawData):
             self.add_data(geoclaw.FixedGridData(),'fixed_grid_data')
             self.add_data(geoclaw.QinitData(),'qinit_data')
             self.add_data(geoclaw.FGmaxData(),'fgmax_data')
-
-            # can remove try-except after PR clawpack/geoclaw#80 is merged
-            try:
-                self.add_data(geoclaw.SurgeData(),'surge_data')
-                self.add_data(geoclaw.FrictionData(),'friction_data')
-            except:
-                pass 
+            self.add_data(geoclaw.SurgeData(),'surge_data')
+            self.add_data(geoclaw.FrictionData(),'friction_data')
+            self.add_data(geoclaw.MultilayerData(), 'multilayer_data')
 
         else:
             raise AttributeError("Unrecognized Clawpack pkg = %s" % pkg)
@@ -386,10 +518,29 @@ class ClawRunData(ClawData):
         return data
 
 
-    def write(self):
+    def write(self, out_dir = ''):
         r"""Write out each data objects in datalist """
+        
+        import clawpack.amrclaw.data as amrclaw
+
         for data_object in self.data_list:
-            data_object.write()
+            # UserData doesn't naturally have an "out_file" parameter
+            if isinstance(data_object, UserData):
+                fname = data_object.__fname__
+            else:
+                if six.PY2:
+                    argspec = inspect.getargspec(data_object.write)
+                    index = argspec.args.index('out_file') - (len(argspec.args) 
+                                                        - len(argspec.defaults))
+                    fname = argspec.defaults[index]
+                else:
+                    argspec = inspect.signature(data_object.write)
+                    fname = argspec.parameters['out_file'].default
+            fpath = os.path.join(out_dir,fname)
+            if isinstance(data_object, amrclaw.GaugeData):
+                data_object.write(self.clawdata.num_eqn, self.clawdata.num_aux, out_file=fpath)
+            else:
+                data_object.write(out_file=fpath)
 
 
 
@@ -526,8 +677,8 @@ class ClawInputData(ClawData):
             iout_q = self.num_eqn * [0]
         else:
             #iout_q = np.where(self.output_q_components, 1, 0)
-            print "*** WARNING: Selective output_q_components not implemented"
-            print "***          Will output all components of q"
+            print("*** WARNING: Selective output_q_components not implemented")
+            print("***          Will output all components of q")
             iout_q = self.num_eqn * [1]
     
 
@@ -535,7 +686,7 @@ class ClawInputData(ClawData):
         self.data_write('', value=iout_q, alt_name='iout_q')
 
         if self.num_aux > 0:
-            if isinstance(self.output_aux_components,basestring):
+            if isinstance(self.output_aux_components,six.string_types):
                 if self.output_aux_components.lower() == 'all':
                     iout_aux = self.num_aux * [1]
                 elif self.output_aux_components.lower() == 'none':
@@ -544,8 +695,8 @@ class ClawInputData(ClawData):
                     raise ValueError("Invalid aux array component option.")
             else:
                 iout_aux = np.where(self.output_aux_components, 1, 0)
-                print "*** WARNING: Selective output_aux_components not implemented"
-                print "***          Will output all components of aux"
+                print("*** WARNING: Selective output_aux_components not implemented")
+                print("***          Will output all components of aux")
                 iout_aux = self.num_aux * [1]
             self.data_write(name='', value=iout_aux, alt_name='iout_aux')
             self.data_write('output_aux_onlyonce')
@@ -585,7 +736,7 @@ class ClawInputData(ClawData):
                 if not (self.transverse_waves in [0, 10, 11, 20, 21, 22]):
                     raise AttributeError("Unrecognized transverse_waves: %s" \
                                              % self.transverse_waves)
-            self.data_write(file, self.transverse_waves, 'transverse_waves')
+            self.data_write(None, self.transverse_waves, 'transverse_waves')
 
             if self.dimensional_split in [0,'unsplit']:  
                 self.dimensional_split = 0
@@ -667,13 +818,13 @@ class ClawInputData(ClawData):
         self.data_write('restart_file')
         self.data_write('checkpt_style')
 
-        if self.checkpt_style==2:
+        if self.checkpt_style in [-2,2]:
             num_checkpt_times = len(self.checkpt_times)
             self.data_write(name='', value=num_checkpt_times, alt_name='num_checkpt_times')
             self.data_write('checkpt_times')
-        elif self.checkpt_style==3:
+        elif self.checkpt_style in [-3,3]:
             self.data_write('checkpt_interval')
-        elif self.checkpt_style not in [0,1]:
+        elif self.checkpt_style not in [0,1,-1]:
             raise AttributeError("*** Unrecognized checkpt_style: %s"\
                   % self.checkpt_style)
 
@@ -704,6 +855,8 @@ class UserData(ClawData):
          descr_dict = self.__descr__
          descr_dict[name] = descr
 
-    def write(self,data_source='setrun.py'):
-        super(UserData,self).write(self.__fname__, data_source)
+    def write(self, data_source='setrun.py', out_file=None):
+        if out_file is None:
+            out_file = self.__fname__
+        super(UserData,self).write(out_file, data_source)
         self.close_data_file()
