@@ -1,17 +1,17 @@
 """
 Module to set up run time parameters for Clawpack.
-
 The values set in the function setrun are then written out to data files
 that will be read in by the Fortran code.
-
 """
 
-import numpy as numpy
+from __future__ import absolute_import
+from __future__ import print_function
+import numpy
 
-import clawpack.geoclaw.multilayer.data as multilayer
-import clawpack.geoclaw.surge.data as surge
+import clawpack.geoclaw.data
 import clawpack.geoclaw.topotools as tt
 
+
 # Rotation transformations
 def transform_c2p(x,y,x0,y0,theta):
     return ((x+x0)*numpy.cos(theta) - (y+y0)*numpy.sin(theta),
@@ -22,19 +22,63 @@ def transform_p2c(x,y,x0,y0,theta):
             -x*numpy.sin(theta) + y*numpy.cos(theta) - y0)
 
 
+# Class containing some setup for the qinit especially for multilayer tests 
+class QinitMultilayerData(clawpack.geoclaw.data.QinitData):
+    r"""
+    Modified Qinit data object for multiple layers
+    """
+
+    def __init__(self):
+        super(QinitMultilayerData, self).__init__()
+
+        # Test qinit data > 4
+        self.add_attribute("init_location", [0.0, 0.0])
+        self.add_attribute("wave_family", 1)
+        self.add_attribute("epsilon", 0.02)
+        self.add_attribute("angle", 0.0)
+        self.add_attribute("sigma", 0.02)
+
+    def write(self, data_source='setrun.py'):
+
+        # Initial perturbation
+        self.open_data_file('qinit.data',data_source)
+        self.data_write('qinit_type')
+
+        # Perturbation requested
+        if self.qinit_type == 0:
+            pass
+        elif 0 < self.qinit_type < 5:
+            # Check to see if each qinit file is present and then write the data
+            for tfile in self.qinitfiles:
+                try:
+                    fname = "'%s'" % os.path.abspath(tfile[-1])
+                except:
+                    raise Warning("File %s was not found." % fname)
+                    # raise MissingFile("file not found")
+                self._out_file.write("\n%s  \n" % fname)
+                self._out_file.write("%3i %3i \n" % tuple(tfile[:-1]))
+        elif self.qinit_type >= 5 and self.qinit_type <= 9:
+            self.data_write('epsilon')
+            self.data_write("init_location")
+            self.data_write("wave_family")
+            self.data_write("angle")
+            self.data_write("sigma")
+        else:
+            raise ValueError("Invalid qinit_type parameter %s." % self.qinit_type)
+        self.close_data_file()
+
+
+
 #------------------------------
 def setrun(claw_pkg='geoclaw'):
 #------------------------------
 
     """
     Define the parameters used for running Clawpack.
-
     INPUT:
         claw_pkg expected to be "geoclaw" for this setrun.
-
     OUTPUT:
         rundata - object of class ClawRunData
-
     """
 
     from clawpack.clawutil import data
@@ -48,13 +92,7 @@ def setrun(claw_pkg='geoclaw'):
     # GeoClaw specific parameters:
     #------------------------------------------------------------------
     rundata = setgeo(rundata)
-
-    rundata.add_data(multilayer.MultilayerData(), 'multilayer_data')
-    set_multilayer(rundata)
-    rundata.add_data(surge.FrictionData(),'frictiondata')
-    set_friction(rundata)
-    rundata.add_data(surge.SurgeData(),'stormdata')
-    set_storm(rundata)
+    rundata = set_multilayer(rundata)
 
     #------------------------------------------------------------------
     # Standard Clawpack parameters to be written to claw.data:
@@ -66,7 +104,6 @@ def setrun(claw_pkg='geoclaw'):
     # Set single grid parameters first.
     # See below for AMR parameters.
 
-
     # ---------------
     # Spatial domain:
     # ---------------
@@ -75,17 +112,15 @@ def setrun(claw_pkg='geoclaw'):
     clawdata.num_dim = num_dim
 
     # Lower and upper edge of computational domain:
-    clawdata.lower[0] = -1      # west longitude
-    clawdata.upper[0] = 2.0       # east longitude
-
-    clawdata.lower[1] = -1.0       # south latitude
-    clawdata.upper[1] = 2.0         # north latitude
-
+    clawdata.lower[0] = -2      # west longitude
+    clawdata.upper[0] = 4.0       # east longitude
 
+    clawdata.lower[1] = -2.0       # south latitude
+    clawdata.upper[1] = 4.0         # north latitude
 
     # Number of grid cells: Coarsest grid
-    clawdata.num_cells[0] = 150
-    clawdata.num_cells[1] = 150
+    clawdata.num_cells[0] = 80
+    clawdata.num_cells[1] = 80
 
     # ---------------
     # Size of system:
@@ -95,27 +130,22 @@ def setrun(claw_pkg='geoclaw'):
     clawdata.num_eqn = 6
 
     # Number of auxiliary variables in the aux array (initialized in setaux)
-    clawdata.num_aux = 4 + rundata.multilayer_data.num_layers
-    if rundata.stormdata.storm_type > 0:
-        clawdata.num_aux += 3
+    # bathy == 1
+    # layer-depths = 2:3
+    clawdata.num_aux = 1 + rundata.multilayer_data.num_layers
 
     # Index of aux array corresponding to capacity function, if there is one:
     clawdata.capa_index = 0
 
-    
-    
     # -------------
     # Initial time:
     # -------------
 
     clawdata.t0 = 0.0
 
-
     # Restart from checkpoint file of a previous run?
-    # Note: If restarting, you must also change the Makefile to set:
-    #    RESTART = True
     # If restarting, t0 above should be from original run, and the
-    # restart_file 'fort.chkNNNNN' specified below should be in 
+    # restart_file 'fort.chkNNNNN' specified below should be in
     # the OUTDIR indicated in Makefile.
 
     clawdata.restart = False               # True to restart from prior results
@@ -133,8 +163,8 @@ def setrun(claw_pkg='geoclaw'):
 
     if clawdata.output_style==1:
         # Output nout frames at equally spaced times up to tfinal:
-        clawdata.num_output_times = 40
-        clawdata.tfinal = 1.0
+        clawdata.num_output_times = 10
+        clawdata.tfinal = 0.3
         clawdata.output_t0 = True  # output at initial (or restart) time?
 
     elif clawdata.output_style == 2:
@@ -270,15 +300,15 @@ def setrun(claw_pkg='geoclaw'):
         # Do not checkpoint at all
         pass
 
-    elif clawdata.checkpt_style == 1:
+    elif numpy.abs(clawdata.checkpt_style) == 1:
         # Checkpoint only at tfinal.
         pass
 
-    elif clawdata.checkpt_style == 2:
+    elif numpy.abs(clawdata.checkpt_style) == 2:
         # Specify a list of checkpoint times.  
         clawdata.checkpt_times = [0.1,0.15]
 
-    elif clawdata.checkpt_style == 3:
+    elif numpy.abs(clawdata.checkpt_style) == 3:
         # Checkpoint every checkpt_interval timesteps (on Level 1)
         # and at the final time.
         clawdata.checkpt_interval = 5
@@ -290,12 +320,12 @@ def setrun(claw_pkg='geoclaw'):
     amrdata = rundata.amrdata
 
     # max number of refinement levels:
-    amrdata.amr_levels_max = 1
+    amrdata.amr_levels_max = 3
 
     # List of refinement ratios at each level (length at least mxnest-1)
-    amrdata.refinement_ratios_x = [2,6]
-    amrdata.refinement_ratios_y = [2,6]
-    amrdata.refinement_ratios_t = [2,6]
+    amrdata.refinement_ratios_x = [2,2]
+    amrdata.refinement_ratios_y = [2,2]
+    amrdata.refinement_ratios_t = [2,2]
 
 
     # Specify type of each aux variable in amrdata.auxtype.
@@ -381,19 +411,19 @@ def setgeo(rundata):
     try:
         geo_data = rundata.geo_data
     except:
-        print "*** Error, this rundata has no geo_data attribute"
+        print("*** Error, this rundata has no geo_data attribute")
         raise AttributeError("Missing geo_data attribute")
-       
+
     # == Physics ==
     geo_data.gravity = 9.81
     geo_data.coordinate_system = 1
     geo_data.earth_radius = 6367.5e3
+    geo_data.rho = [922.5, 1025.0]
 
     # == Forcing Options
     geo_data.coriolis_forcing = False
 
     # == Algorithm and Initial Conditions ==
-    geo_data.sea_level = 0.0
     geo_data.dry_tolerance = 1.e-3
     geo_data.friction_forcing = True
     geo_data.manning_coefficient = 0.025
@@ -411,28 +441,16 @@ def setgeo(rundata):
     # for topography, append lines of the form
     #    [topotype, minlevel, maxlevel, t1, t2, fname]
     topo_data.topofiles.append([2, 1, 5, 0.0, 1e10, 'topo.tt2'])
-    
+
     # == setdtopo.data values ==
     dtopo_data = rundata.dtopo_data
-    # for moving topography, append lines of the form :   (<= 1 allowed for now!)
+    # for moving topography, append lines of the form : (<= 1 allowed for now!)
     #   [topotype, minlevel,maxlevel,fname]
 
-    # == setqinit.data values ==
-    # Use multilayer qinit data object
-
-
-
     return rundata
     # end of function setgeo
     # ----------------------
 
-def set_friction(rundata):
-
-    data = rundata.frictiondata
-
-    # Variable friction
-    data.variable_friction = False
-
 
 def set_multilayer(rundata):
 
@@ -440,62 +458,56 @@ def set_multilayer(rundata):
 
     # Physics parameters
     data.num_layers = 2
-    data.rho = [0.9,1.0]
-    data.eta = [0.0,-0.6]
-    
+    data.eta = [0.0, -0.6]
+    data.rho = [922.5, 1025.0]
+
     # Algorithm parameters
     data.eigen_method = 2
     data.inundation_method = 2
     data.richardson_tolerance = 0.95
-    # data.wave_tolerance = [0.1,0.1]
-    # data.dry_limit = True
+    data.wave_tolerance = [1e-3, 1e-2]
+    data.layer_index = 1
 
-    # Set special initial conditions for qinit
-    rundata.replace_data('qinit_data', multilayer.QinitMultilayerData())
+    rundata.replace_data('qinit_data', QinitMultilayerData())
     rundata.qinit_data.qinit_type = 6
     rundata.qinit_data.epsilon = 0.02
     rundata.qinit_data.angle = 0.0
     rundata.qinit_data.sigma = 0.02
     rundata.qinit_data.wave_family = 4
-    rundata.qinit_data.init_location = [-0.1,0.0]
+    rundata.qinit_data.init_location = [-0.1, 0.0]
 
-def set_storm(rundata):
-
-    # No storm
-    rundata.stormdata.storm_type = 0
+    return rundata
 
 
 def bathy_step(x, y, location=0.15, angle=0.0, left=-1.0, right=-0.2):
-    x_c,y_c = transform_p2c(x, y, location, 0.0, angle)
-    return ((x_c <= 0.0) * left 
-          + (x_c >  0.0) * right)
+    x_c, y_c = transform_p2c(x, y, location, 0.0, angle)
+    return ((x_c <= 0.0) * left +
+            (x_c > 0.0) * right)
 
 
 def write_topo_file(run_data, out_file, **kwargs):
-
-
     # Make topography
     topo_func = lambda x, y: bathy_step(x, y, **kwargs)
     topo = tt.Topography(topo_func=topo_func)
-    topo.x = numpy.linspace(run_data.clawdata.lower[0], 
-                            run_data.clawdata.upper[0], 
+    topo.x = numpy.linspace(run_data.clawdata.lower[0],
+                            run_data.clawdata.upper[0],
                             run_data.clawdata.num_cells[0] + 8)
-    topo.y = numpy.linspace(run_data.clawdata.lower[1], 
-                            run_data.clawdata.upper[1], 
+    topo.y = numpy.linspace(run_data.clawdata.lower[1],
+                            run_data.clawdata.upper[1],
                             run_data.clawdata.num_cells[1] + 8)
     topo.write(out_file)
 
     # Write out simple bathy geometry file for communication to the plotting
     with open("./bathy_geometry.data", 'w') as bathy_geometry_file:
-        if kwargs.has_key("location"):
+        if "location" in kwargs:
             location = kwargs['location']
         else:
             location = 0.15
-        if kwargs.has_key("angle"):
+        if "angle" in kwargs:
             angle = kwargs['angle']
         else:
             angle = 0.0
-        bathy_geometry_file.write("%s\n%s" % (location, angle) )
+        bathy_geometry_file.write("%s\n%s" % (location, angle))
 
 
 if __name__ == '__main__':
@@ -508,4 +520,4 @@ if __name__ == '__main__':
 
     rundata.write()
 
-    write_topo_file(rundata, 'topo.tt2')
\ No newline at end of file
+    write_topo_file(rundata, 'topo.tt2')
