@@ -79,20 +79,33 @@ program amr2
     use amr_module, only: output_aux_onlyonce, matlabu
 
     use amr_module, only: lfine, lentot, iregridcount, avenumgrids
-    use amr_module, only: tvoll, rvoll, rvol, mstart, possk, ibuff
-    use amr_module, only: timeRegridding, timeUpdating, timeValout
-    use amr_module, only: timeGrdfitAll,timeFlglvl,timeGrdfit2,timeSetaux,timeFilval
-    use amr_module, only: timeBound, timeStepgrid, timeFlagger,timeBufnst,timeFilvalTot
+    use amr_module, only: tvoll, tvollCPU, rvoll, rvol, mstart, possk, ibuff
+    use amr_module, only: timeRegridding,timeUpdating, timeValout
+    use amr_module, only: timeBound,timeStepgrid, timeFlagger,timeBufnst
+    use amr_module, only: timeBoundCPU,timeStepGridCPU,timeRegriddingCPU
+    use amr_module, only: timeValoutCPU,timeTick,timeTickCPU
     use amr_module, only: kcheck, iorder, lendim, lenmax
 
     use amr_module, only: dprint, eprint, edebug, gprint, nprint, pprint
     use amr_module, only: rprint, sprint, tprint, uprint
 
     use amr_module, only: t0, tstart_thisrun
+    use amr_module, only: tick_clock_start, tick_cpu_start
 
-    use regions_module, only: set_regions
+
+    ! Data modules
+    use geoclaw_module, only: set_geo
+    use topo_module, only: read_topo_settings, read_dtopo_settings
+    use qinit_module, only: set_qinit
+    use fixedgrids_module, only: set_fixed_grids
+    use refinement_module, only: set_refinement
+    use storm_module, only: set_storm
+    use friction_module, only: setup_variable_friction
     use gauges_module, only: set_gauges, num_gauges
+    use regions_module, only: set_regions
     use fgmax_module, only: set_fgmax, FG_num_fgrids
+    use multilayer_module, only: set_multilayer
+    use adjoint_module, only: read_adjoint_data
 
     implicit none
 
@@ -103,10 +116,18 @@ program amr2
     integer :: omp_get_max_threads, maxthreads
     real(kind=8) :: time, ratmet, cut, dtinit, dt_max
     logical :: vtime, rest, output_t0    
-    integer :: num_fgmax
 
     ! Timing variables
-    integer :: clock_start, clock_finish, clock_rate
+    integer(kind=8) :: clock_start, clock_finish, clock_rate, ttotal, count_max
+    real(kind=8) :: ttotalcpu
+    integer(kind=8) :: tick_clock_finish
+    integer, parameter :: timing_unit = 48
+    character(len=512) :: timing_line, timing_substr
+    character(len=*), parameter :: timing_base_name = "timing."
+    character(len=*), parameter :: timing_header_format =                      &
+                                                  "(' wall time (', i2,')," // &
+                                                  " CPU time (', i2,'), "   // &
+                                                  "cells updated (', i2,'),')"
 
     ! Common block variables
     real(kind=8) :: dxmin, dymin
@@ -163,7 +184,7 @@ program amr2
         read(inunit,*) nout
         allocate(tout(nout))
         read(inunit,*) (tout(i), i=1,nout)
-        output_t0 = (tout(1) == t0)
+        output_t0 = (abs(tout(1) - t0) < 1e-15)
         ! Move output times down one index
         if (output_t0) then
             nout = nout - 1
@@ -293,12 +314,12 @@ program amr2
         ! Never checkpoint:
         checkpt_interval = iinfinity
 
-    else if (checkpt_style == 2) then
+    else if (abs(checkpt_style) == 2) then
         read(inunit,*) nchkpt
         allocate(tchk(nchkpt))
         read(inunit,*) (tchk(i), i=1,nchkpt)
 
-    else if (checkpt_style == 3) then
+    else if (abs(checkpt_style) == 3) then
         ! Checkpoint every checkpt_interval steps on coarse grid
         read(inunit,*) checkpt_interval
     endif
@@ -366,13 +387,6 @@ program amr2
     ! Finished with reading in parameters
     ! ==========================================================================
 
-    ! Read in region and gauge data
-    call set_regions('regions.data')
-    call set_gauges('gauges.data')
-
-    ! New fixed grid routines to keep track of max over computation:
-    call set_fgmax('fgmax.data')
-
     ! Look for capacity function via auxtypes:
     mcapa = 0
     do iaux = 1, naux
@@ -446,6 +460,8 @@ program amr2
         open(outunit, file=outfile, status='unknown', position='append', &
                       form='formatted')
 
+        ! moved upt before restrt or won't properly initialize 
+        call set_fgmax()   
         call restrt(nsteps,time,nvar,naux)
         nstart  = nsteps
         tstart_thisrun = time
@@ -456,8 +472,35 @@ program amr2
         ! Call user routine to set up problem parameters:
         call setprob()
 
+        ! Non-user defined setup routine
+        call set_geo()                    ! sets basic parameters g and coord system
+        call set_refinement()             ! sets refinement control parameters
+        call read_dtopo_settings()        ! specifies file with dtopo from earthquake
+        call read_topo_settings()         ! specifies topography (bathymetry) files
+        call set_qinit()                  ! specifies file with dh if this used instead
+        call set_fixed_grids()            ! Fixed grid settings
+        call setup_variable_friction()    ! Variable friction parameter
+        call set_multilayer()             ! Set multilayer SWE parameters
+        call set_storm()                  ! Set storm parameters
+        call set_regions()                ! Set refinement regions
+        call set_gauges(rest, nvar, naux) ! Set gauge output
+        call read_adjoint_data()          ! Read adjoint solution
+
     else
 
+        ! Create new timing file
+        open(unit=timing_unit, file=timing_base_name//"csv",                &
+             form='formatted', status='unknown', action='write')
+        ! Construct header string
+        timing_line = 'output_time,total_wall_time,total_cpu_time,'
+        timing_substr = ""
+        do level=1, mxnest
+            write(timing_substr, timing_header_format) level, level, level
+            timing_line = trim(timing_line) // trim(timing_substr)
+        end do
+        write(timing_unit, "(a)") timing_line
+        close(timing_unit)
+
         open(outunit, file=outfile, status='unknown', form='formatted')
 
         tstart_thisrun = t0
@@ -465,6 +508,21 @@ program amr2
         ! Call user routine to set up problem parameters:
         call setprob()
 
+        ! Non-user defined setup routine
+        call set_geo()                    ! sets basic parameters g and coord system
+        call set_refinement()             ! sets refinement control parameters
+        call read_dtopo_settings()        ! specifies file with dtopo from earthquake
+        call read_topo_settings()         ! specifies topography (bathymetry) files
+        call set_qinit()                  ! specifies file with dh if this used instead
+        call set_fixed_grids()            ! Fixed grid settings
+        call setup_variable_friction()    ! Variable friction parameter
+        call set_multilayer()             ! Set multilayer SWE parameters
+        call set_storm()                  ! Set storm parameters
+        call set_regions()                ! Set refinement regions
+        call set_gauges(rest, nvar, naux) ! Set gauge output
+        call set_fgmax()
+        call read_adjoint_data()          ! Read adjoint solution
+
         cflmax = 0.d0   ! otherwise use previously heckpointed val
 
         lentot = 0
@@ -556,15 +614,13 @@ program amr2
     print *, 'Done reading data, starting computation ...  '
     print *, ' '
 
-
+    ! initialize timers before calling valout:
+    call system_clock(tick_clock_start, clock_rate)
+    call cpu_time(tick_cpu_start)
 
     call outtre (mstart,printout,nvar,naux)
     write(outunit,*) "  original total mass ..."
     call conck(1,nvar,naux,time,rest)
-
-   ! Timing
-    call system_clock(clock_start,clock_rate)
-
     if (output_t0) then
         call valout(1,lfine,time,nvar,naux)
     endif
@@ -582,72 +638,144 @@ program amr2
 
     ! Print out the fgmax files
     if (FG_num_fgrids > 0) call fgmax_finalize()
+    
+    ! call system_clock to get clock_finish and count_max for debug output:
+    call system_clock(clock_finish,clock_rate,count_max)
+    
+    !output timing data
+    open(timing_unit, file=timing_base_name//"txt", status='unknown',       &
+         form='formatted')
+    write(*,*)
+    write(timing_unit,*)
+    format_string="('============================== Timing Data ==============================')"
+    write(timing_unit,format_string)
+    write(*,format_string)
 
+    write(*,*)
+    write(timing_unit,*)
+    
+    !Integration time
+    format_string="('Integration Time (stepgrid + BC + overhead)')"
+    write(timing_unit,format_string)
+    write(*,format_string)
+    
+    !Advanc time
+    format_string="('Level           Wall Time (seconds)    CPU Time (seconds)   Total Cell Updates')"
+    write(timing_unit,format_string)
+    write(*,format_string)
 
-    call system_clock(clock_finish,clock_rate)
-    format_string = "('Total time to solution = ',1f16.8,' s, using', i3,' threads')"
-    write(outunit,format_string) &
-            real(clock_finish - clock_start,kind=8) / real(clock_rate,kind=8), maxthreads
-    write(*,format_string) &
-            real(clock_finish - clock_start,kind=8) / real(clock_rate,kind=8), maxthreads
-
-    do level = 1, mxnest            
-      format_string = "('Total advanc time on level ',i3,' = ',1f16.8,' s')"
-      write(outunit,format_string) level, &
-             real(tvoll(level),kind=8) / real(clock_rate,kind=8)
-      write(*,format_string) level, &
-             real(tvoll(level),kind=8) / real(clock_rate,kind=8)
+    ! level counters are cumulative after restart, so initialize sums to zero 
+    ! even after restart to sum up time over all levels
+    ttotalcpu=0.d0
+    ttotal=0
+      
+    do level=1,mxnest
+        format_string="(i3,'           ',1f15.3,'        ',1f15.3,'    ', e17.3)"
+        write(timing_unit,format_string) level, &
+             real(tvoll(level),kind=8) / real(clock_rate,kind=8), tvollCPU(level), rvoll(level)
+        write(*,format_string) level, &
+             real(tvoll(level),kind=8) / real(clock_rate,kind=8), tvollCPU(level), rvoll(level)
+        ttotalcpu=ttotalcpu+tvollCPU(level)
+        ttotal=ttotal+tvoll(level)
     end do
-    format_string = "('Total updating   time            ',1f16.8,' s')"
-    write(outunit,format_string)  real(timeUpdating,kind=8) / real(clock_rate,kind=8)
-    write(*,format_string) real(timeUpdating,kind=8) / real(clock_rate,kind=8)
-    format_string = "('Total valout     time            ',1f16.8,' s')"
-    write(outunit,format_string)  real(timeValout,kind=8) / real(clock_rate,kind=8)
-    write(*,format_string) real(timeValout,kind=8) / real(clock_rate,kind=8)
+    
+    format_string="('total         ',1f15.3,'        ',1f15.3,'    ', e17.3)"
+    write(timing_unit,format_string) &
+             real(ttotal,kind=8) / real(clock_rate,kind=8), ttotalCPU, rvol
+    write(*,format_string) &
+             real(ttotal,kind=8) / real(clock_rate,kind=8), ttotalCPU, rvol
+    
+    write(*,*)
+    write(timing_unit,*)
+    
+    
+    format_string="('All levels:')"
+    write(*,format_string)
+    write(timing_unit,format_string)
+    
+    !stepgrid
+    format_string="('stepgrid      ',1f15.3,'        ',1f15.3,'    ',e17.3)"
+    write(timing_unit,format_string) &
+         real(timeStepgrid,kind=8) / real(clock_rate,kind=8), timeStepgridCPU
+    write(*,format_string) &
+         real(timeStepgrid,kind=8) / real(clock_rate,kind=8), timeStepgridCPU
+    
+    !bound
+    format_string="('BC/ghost cells',1f15.3,'        ',1f15.3)"
+    write(timing_unit,format_string) &
+         real(timeBound,kind=8) / real(clock_rate,kind=8), timeBoundCPU
+    write(*,format_string) &
+         real(timeBound,kind=8) / real(clock_rate,kind=8), timeBoundCPU
+    
+    !regridding time
+    format_string="('Regridding    ',1f15.3,'        ',1f15.3,'  ')"
+    write(timing_unit,format_string) &
+            real(timeRegridding,kind=8) / real(clock_rate,kind=8), timeRegriddingCPU
+    write(*,format_string) &
+            real(timeRegridding,kind=8) / real(clock_rate,kind=8), timeRegriddingCPU
+    
+    !output time
+    format_string="('Output (valout)',1f14.3,'        ',1f15.3,'  ')"
+    write(timing_unit,format_string) &
+            real(timeValout,kind=8) / real(clock_rate,kind=8), timeValoutCPU
+    write(*,format_string) &
+            real(timeValout,kind=8) / real(clock_rate,kind=8), timeValoutCPU
+    
+    write(*,*)
+    write(timing_unit,*)
+    
+    !Total Time
+    format_string="('Total time:   ',1f15.3,'        ',1f15.3,'  ')"
+
+    write(*,format_string) real(timeTick,kind=8)/real(clock_rate,kind=8), &
+            timeTickCPU
+    write(timing_unit,format_string) real(timeTick,kind=8)/real(clock_rate,kind=8), &
+            timeTickCPU
+
+    
+    format_string="('Using',i3,' thread(s)')"
+    write(timing_unit,format_string) maxthreads
+    write(*,format_string) maxthreads
+    
+    
+    write(*,*)
+    write(timing_unit,*)
+    
+    
+    write(*,"('Note: The CPU times are summed over all threads.')")
+    write(timing_unit,"('Note: The CPU times are summed over all threads.')")
+    write(*,"('      Total time includes more than the subroutines listed above')")
+    write(timing_unit,"('      Total time includes more than the subroutines listed above')")
+    if (rest) then
+      write(*,"('      Times for restart runs are cumulative')")
+      write(timing_unit,"('      Times for restart runs are cumulative')")
+    endif
+
+    write(*, "('Note: timings are also recorded for each output step')")
+    write(*, "('      in the file timing.csv.')")
+    write(timing_unit, "('Note: timings are also recorded for each output step')")
+    write(timing_unit, "('      in the file timing.csv.')")
+    
 
     write(*,*)
-    format_string = "('Total regridding time (clock)    ',1f16.8,' s')"
-    write(outunit,format_string)  real(timeRegridding,kind=8) / real(clock_rate,kind=8)
-    write(*,format_string) real(timeRegridding,kind=8) / real(clock_rate,kind=8)
- 
-    format_string = "('   Total Grdfit     time (clock)     ',1f16.8,' s')"
-    write(outunit,format_string)  real(timeGrdfitAll,kind=8) / real(clock_rate,kind=8)
-    write(*,format_string) real(timeGrdfitAll,kind=8) / real(clock_rate,kind=8)
-   format_string = "('      Total Flglvl  time                ',1f16.8,' s')"
-    write(outunit,format_string)  real(timeFlglvl,kind=8) / real(clock_rate,kind=8)
-    write(*,format_string) real(timeFlglvl,kind=8) / real(clock_rate,kind=8)
-    format_string = "('         Total    Flagger     time (clock)     ',1f16.8,' s')"
-    write(outunit,format_string)  real(timeFlagger,kind=8) / real(clock_rate,kind=8)
-    write(*,format_string) real(timeFlagger,kind=8) / real(clock_rate,kind=8)
-    format_string = "('         Total    Bufnst     time (clock)     ',1f16.8,' s')"
-    write(outunit,format_string)  real(timeBufnst,kind=8) / real(clock_rate,kind=8)
-    write(*,format_string) real(timeBufnst,kind=8) / real(clock_rate,kind=8)
+    write(timing_unit,*)
 
-    format_string = "('   Total gfixup     time  (clock)   ',1f16.8,' s')"
-    write(outunit,format_string)  real(timeGrdfit2,kind=8) / real(clock_rate,kind=8)
-    write(*,format_string) real(timeGrdfit2,kind=8) / real(clock_rate,kind=8)
-    format_string = "('      Total filval (wall) time      ',1f16.8,' s')"
-    write(outunit,format_string)  real(timeFilvalTot,kind=8) / real(clock_rate,kind=8)
-    write(*,format_string) real(timeFilvalTot,kind=8) / real(clock_rate,kind=8)
+    ! output clock_rate etc. useful in debugging or if negative time reported
 
-   format_string = "('          Total filval (all cores) time     ',1f16.8,' s')"
-    write(outunit,format_string)  real(timeFilval,kind=8) / real(clock_rate,kind=8)
-    write(*,format_string) real(timeFilval,kind=8) / real(clock_rate,kind=8)
+    format_string="('clock_rate = ',i10, ' per second,  count_max = ',i23)"
+    !write(*,format_string) clock_rate, count_max
+    write(timing_unit,format_string) clock_rate, count_max
 
+    format_string="('clock_start = ',i20, ',  clock_finish = ',i20)"
+    !write(*,format_string) tick_clock_start, clock_finish
+    write(timing_unit,format_string) tick_clock_start, clock_finish
+
+    format_string="('=========================================================================')"
+    write(timing_unit,format_string)
+    write(*,format_string)
     write(*,*)
-    format_string = "('Total setaux (all cores) time     ',1f16.8,' s')"
-    write(outunit,format_string)  real(timeSetaux,kind=8) / real(clock_rate,kind=8)
-    write(*,format_string) real(timeSetaux,kind=8) / real(clock_rate,kind=8)
-
-
-    write(*,*)
-    write(*,*)" integration time, still not counting saveqc"
-    format_string = "('Total Bound (all levels) wall time      ',1f16.8,' s')"
-    write(outunit,format_string)  real(timeBound,kind=8) / real(clock_rate,kind=8)
-    write(*,format_string) real(timeBound,kind=8) / real(clock_rate,kind=8)
-    format_string = "('Total stepgrid (all levels) wall time   ',1f16.8,' s')"
-    write(outunit,format_string)  real(timeStepgrid,kind=8) / real(clock_rate,kind=8)
-    write(*,format_string) real(timeStepgrid,kind=8) / real(clock_rate,kind=8)
+    write(timing_unit,*)
+    close(timing_unit)
 
     ! Done with computation, cleanup:
     lentotsave = lentot
