@@ -12,13 +12,13 @@
 !
 ! :::::::::::::::::::::::::::::::::::::::;:::::::::::::::::::::::;
 recursive subroutine filrecur(level,nvar,valbig,aux,naux,t,mx,my, &
-                              nrowst,ncolst,ilo,ihi,jlo,jhi)
-!                              nrowst,ncolst,fill_indices)
+                              nrowst,ncolst,ilo,ihi,jlo,jhi,patchOnly,msrc,  &
+                              do_aux_copy)
 
     use amr_module, only: nghost, xlower, ylower, xupper, yupper, outunit
     use amr_module, only: xperdom, yperdom, spheredom, hxposs, hyposs
     use amr_module, only: intratx, intraty, iregsz, jregsz
-    use amr_module, only: timeSetaux, NEEDS_TO_BE_SET
+    use amr_module, only: NEEDS_TO_BE_SET
 
     use geoclaw_module, only: sea_level, dry_tolerance
     use topo_module, only: topo_finalized
@@ -27,16 +27,15 @@ recursive subroutine filrecur(level,nvar,valbig,aux,naux,t,mx,my, &
 
     ! Input
     integer, intent(in) :: level, nvar, naux, mx, my, nrowst, ncolst
-    integer, intent(in) :: ilo,ihi,jlo,jhi
-!    integer, intent(in) :: fill_indices(4)
+    integer, intent(in) :: ilo,ihi,jlo,jhi,msrc
     real(kind=8), intent(in) :: t
+    logical  :: patchOnly, do_aux_copy, yes_do_aux_copy
 
     ! Output
     real(kind=8), intent(in out) :: valbig(nvar,mx,my)
     real(kind=8), intent(in out) :: aux(naux,mx,my)
 
     ! Local storage
-    integer  :: clock_start, clock_finish, clock_rate, clock_dif
     integer  :: iplo, iphi, jplo, jphi
 
     ! Flagging of set cells
@@ -46,25 +45,20 @@ recursive subroutine filrecur(level,nvar,valbig,aux,naux,t,mx,my, &
     integer :: unset_indices(4), coarse_indices(4)
     integer :: refinement_ratio_x, refinement_ratio_y
     real(kind=8) :: dx_fine, dy_fine, dx_coarse, dy_coarse
-    real(kind=8) :: xlow_coarse,ylow_coarse, xlow_fine, ylow_fine, xhi_fine,yhi_fine   !coarse_rect(4), fill_rect(4)
+    real(kind=8) :: xlow_coarse,ylow_coarse, xlow_fine, ylow_fine, xhi_fine,yhi_fine   
     real(kind=8) :: h, b, eta_fine, eta1, eta2, up_slope, down_slope
     real(kind=8) :: hv_fine, v_fine, v_new, divide_mass
     real(kind=8) :: h_fine_average, h_fine, h_count, h_coarse
-!    integer(kind=1) :: flaguse(fill_indices(2)-fill_indices(1)+1,fill_indices(4)-fill_indices(3)+1)
+    real(kind=8)::  xcent_fine,xcent_coarse,ycent_fine,ycent_coarse,ratio_x,ratio_y
     integer(kind=1) :: flaguse(ihi-ilo+1, jhi-jlo+1)
 
+    real(kind=8) :: eta1old, eta2old
+
     ! Scratch arrays for interpolation
-!    logical :: fine_flag(nvar, fill_indices(2) - fill_indices(1) + 2, fill_indices(4) - fill_indices(3) + 2)
     logical :: fine_flag(nvar, ihi-ilo+2,jhi-jlo + 2)
 
     logical :: reloop
 
-!!$    real(kind=8) :: fine_mass(fill_indices(2) - fill_indices(1) + 2, fill_indices(4) - fill_indices(3) + 2)
-!!$    real(kind=8) :: eta_coarse(fill_indices(2) - fill_indices(1) + 2, fill_indices(4) - fill_indices(3) + 2)
-!!$    real(kind=8) :: vel_max(fill_indices(2) - fill_indices(1) + 2, fill_indices(4) - fill_indices(3) + 2)
-!!$    real(kind=8) :: vel_min(fill_indices(2) - fill_indices(1) + 2, fill_indices(4) - fill_indices(3) + 2)
-!!$    real(kind=8) :: slope(2, fill_indices(2) - fill_indices(1) + 2, fill_indices(4) - fill_indices(3) + 2)
-!!$    integer :: fine_cell_count(fill_indices(2)-fill_indices(1)+2, fill_indices(4)-fill_indices(3)+2)
 
     ! these are dimensioned at fine size since coarse grid size cant be larger (incl. the +3 that is )
     real(kind=8) ::  fine_mass(ihi-ilo + 3, jhi-jlo + 3)
@@ -85,13 +79,10 @@ recursive subroutine filrecur(level,nvar,valbig,aux,naux,t,mx,my, &
     !  when pass it in to subroutines they treat it as having di_fineerent
     !  dimensions than the max size need to allocate here
     ! the +2 is to expand on coarse grid to enclose fine
-!!$    real(kind=8) :: valcrse((fill_indices(2)-fill_indices(1)+2) * (fill_indices(4)-fill_indices(3)+2) * nvar)  
-!!$    real(kind=8) :: auxcrse((fill_indices(2)-fill_indices(1)+2) * (fill_indices(4)-fill_indices(3)+2) * num_aux)  
-    real(kind=8) :: valcrse((ihi-ilo+3) * (jhi-jlo+3) * nvar)  
+    real(kind=8) :: valcrse((ihi-ilo+3) * (jhi-jlo+3) * nvar)   ! NB this is a 1D array 
     real(kind=8) :: auxcrse((ihi-ilo+3) * (jhi-jlo+3) * naux)  
-    ! We begin by filling values for grids at level level.
-!!$    mx_patch = fill_indices(2) - fill_indices(1) + 1 ! nrowp
-!!$    my_patch = fill_indices(4) - fill_indices(3) + 1 ! ncolp
+ 
+    yes_do_aux_copy = .true.
 
     mx_patch = ihi-ilo + 1 ! nrowp
     my_patch = jhi-jlo + 1 
@@ -100,19 +91,15 @@ recursive subroutine filrecur(level,nvar,valbig,aux,naux,t,mx,my, &
     dy_fine     = hyposs(level)
 
     ! Coordinates of edges of patch (xlp,xrp,ybp,ytp)
-!!$    fill_rect = [xlower + ilo * dx_fine, &
-!!$                 xlower + (ihi + 1) * dx_fine, &
-!!$                 ylower + jlo * dy_fine, &
-!!$                 ylower + (jhi + 1) * dy_fine]
-
     xlow_fine = xlower + ilo * dx_fine
     ylow_fine = ylower + jlo * dy_fine
 
     ! Fill in the patch as much as possible using values at this level
-!!$    call intfil(valbig,mx,my,t,flaguse,nrowst,ncolst,fill_indices(1), &
-!!$                fill_indices(2),fill_indices(3),fill_indices(4),level,nvar,naux)
-    call intfil(valbig,mx,my,t,flaguse,nrowst,ncolst, ilo,  &
-                ihi,jlo,jhi,level,nvar,naux)
+    ! note that if only a patch, msrc = -1, otherwise a real grid and intfil
+    ! uses its boundary list
+    ! msrc either -1 (for a patch) or the real grid number
+    call intfil(valbig,aux,mx,my,t,flaguse,nrowst,ncolst, ilo,  &
+                ihi,jlo,jhi,level,nvar,naux,msrc,do_aux_copy)
 
     ! Trimbd returns set = true if all of the entries are filled (=1.).
     ! set = false, otherwise. If set = true, then no other levels are
@@ -168,10 +155,7 @@ recursive subroutine filrecur(level,nvar,valbig,aux,naux,t,mx,my, &
                                                 / refinement_ratio_y - nghost
         jphi = (unset_indices(4) + refinement_ratio_y) / refinement_ratio_y
                                                 
-!!$     coarse_rect = [xlower +iplo * dx_coarse, &
-!!$                       xlower + (iphi + 1) * dx_coarse, &
-!!$                       ylower + jplo * dy_coarse, &
-!!$                       ylower + (jphi + 1) * dy_coarse]
+
         xlow_coarse = xlower + iplo * dx_coarse
         ylow_coarse = ylower + jplo * dy_coarse
 
@@ -196,36 +180,42 @@ recursive subroutine filrecur(level,nvar,valbig,aux,naux,t,mx,my, &
         
         if (naux > 0) then
             nghost_patch = 0
-            call system_clock(clock_start,clock_rate)
+            lencrse = (ihi-ilo+3)*(jhi-jlo+3)*naux ! set 1 component, not all naux
+            ! new system checks initialization before setting aux vals
+            ! set here and check later if not all set by copying in intfil
+            ! after some more testing shoudl move thiese auxcrse initialization
+            ! into the branches that need it (topo_finalized dand periodic
+            ! but out of the most used filrecur pathway
+            do k = 1, lencrse, naux
+              auxcrse(k) = NEEDS_TO_BE_SET  
+            end do
   
             ! update topography if needed
             !if ((num_dtopo>0).and.(topo_finalized.eqv..false.)) then
             !   if ((minval(topotime)<maxval(tfdtopo)).and.(t>=minval(t0dtopo))) then
             if (.not. topo_finalized) then
                 call topo_update(t)
-                endif
-
-            nghost_patch = 0                           
-            lencrse = (ihi-ilo+3)*(jhi-jlo+3)*naux ! set 1 component, not all naux
-            do k = 1, lencrse, naux
-              auxcrse(k) = NEEDS_TO_BE_SET  ! new system checks initialization before setting aux vals
-            end do
-            call setaux(nghost_patch, mx_coarse, my_coarse,       &
-                        xlow_coarse, ylow_coarse,                 &
-                        dx_coarse,dy_coarse,naux,auxcrse)
-            call system_clock(clock_finish,clock_rate)
-            clock_dif = clock_finish - clock_start
-!$OMP ATOMIC            
-            timeSetaux = timeSetaux + clock_dif
+                call setaux(nghost_patch, mx_coarse, my_coarse,       &
+                            xlow_coarse, ylow_coarse,                 &
+                            dx_coarse,dy_coarse,naux,auxcrse)
+                yes_do_aux_copy = .false. !grids may not have latest topo
+            endif
         endif
 
         ! Fill in the edges of the coarse grid
         if ((xperdom .or. (yperdom .or. spheredom)) .and. sticksout(iplo,iphi,jplo,jphi)) then
+            call setaux(nghost_patch, mx_coarse, my_coarse,       &
+                        xlow_coarse, ylow_coarse,                 &
+                        dx_coarse,dy_coarse,naux,auxcrse)
             call prefilrecur(level-1,nvar,valcrse,auxcrse,naux,t, &
-                             mx_coarse,my_coarse,1,1,iplo,iphi,jplo,jphi)
-        else
+                             mx_coarse,my_coarse,1,1,iplo,iphi,jplo,jphi,iplo,iphi,jplo,jphi,.true.)
+        else ! when going to coarser patch, no source grid (for now at least) hence -1
             call filrecur(level-1,nvar,valcrse,auxcrse,naux,t,  &
-                          mx_coarse,my_coarse,1,1,iplo,iphi,jplo,jphi)
+                          mx_coarse,my_coarse,1,1,iplo,iphi,jplo,jphi,.true.,-1,    &
+                          yes_do_aux_copy)
+            !call setaux(nghost_patch, mx_coarse, my_coarse,       &
+            !            xlow_coarse, ylow_coarse,                 &
+            !            dx_coarse,dy_coarse,naux,auxcrse)
         endif
 
         ! loop through coarse cells determining intepolation slopes
@@ -274,25 +264,37 @@ recursive subroutine filrecur(level,nvar,valbig,aux,naux,t,mx,my, &
             enddo
         enddo
 
+        ratio_y = real(refinement_ratio_y,kind=8)  ! needs to be real for "floor" call below
+        ratio_x = real(refinement_ratio_x,kind=8)
         ! Loop through patch to be filled, includes multiple coarse cells
          do j_fine = 1, my_patch
-            j_coarse = 2 + (j_fine - (unset_indices(3) - jlo) - 1) / refinement_ratio_y
-            eta2 = (-0.5d0 + real(mod(j_fine - 1, refinement_ratio_y),kind=8)) &
-                                    / real(refinement_ratio_y,kind=8)
+            j_coarse     = floor((j_fine + jlo - 1) / ratio_y) - jplo + 1
+            ycent_coarse = ylow_coarse + (j_coarse-.5d0)*dy_coarse
+            ycent_fine   = ylower + (j_fine-1+jlo + .5d0)*dy_fine
+            eta2         = (ycent_fine-ycent_coarse)/dy_coarse
+            if (abs(eta2) .gt. .5d0) then
+                write(*,*)" filpatch y indexing error: eta2 = ",eta2
+            endif
+            !eta2old = (-0.5d0 + real(mod(j_fine - 1, refinement_ratio_y),kind=8)) &
+            !                        / real(refinement_ratio_y,kind=8)
             do i_fine = 1, mx_patch
-                i_coarse = 2 + (i_fine - (unset_indices(1) - ilo) - 1) / refinement_ratio_x
-                eta1 = (-0.5d0 + real(mod(i_fine - 1, refinement_ratio_x),kind=8)) &
-                                / real(refinement_ratio_x,kind=8)
-           
+                i_coarse     = floor((i_fine+ilo-1) / ratio_x) - iplo + 1
+                xcent_coarse = xlow_coarse + (i_coarse-.5d0)*dx_coarse
+                xcent_fine   =  xlower + (i_fine-1+ilo + .5d0)*dx_fine
+                eta1         = (xcent_fine-xcent_coarse)/dx_coarse
+                if (abs(eta1) .gt. .5d0) then
+                   write(*,*)" filpatch x indexing error: eta1 = ",eta1
+                endif
+                !eta1old = (-0.5d0 + real(mod(i_fine - 1, refinement_ratio_x),kind=8)) &
+                !                / real(refinement_ratio_x,kind=8)
 
                 if (flaguse(i_fine,j_fine) == 0) then
                     ! Interpolate from coarse cells to fine grid for surface
-                    fine_cell_count(i_coarse,j_coarse) = &
-                                        fine_cell_count(i_coarse,j_coarse) + 1
+                    fine_cell_count(i_coarse,j_coarse) = fine_cell_count(i_coarse,j_coarse) + 1
                     eta_fine = eta_coarse(i_coarse,j_coarse) + eta1 * slope(1,i_coarse,j_coarse) &
                                                              + eta2 * slope(2,i_coarse,j_coarse)
                     h_fine = max(eta_fine - aux(1,i_fine + nrowst - 1, j_fine + ncolst - 1), 0.d0)
-                    valbig(1,i_fine + nrowst - 1, j_fine + ncolst - 1) = h_fine
+                    valbig(1,i_fine+nrowst-1, j_fine+ncolst-1) = h_fine
                     fine_mass(i_coarse,j_coarse) = fine_mass(i_coarse,j_coarse) + h_fine
 
                     ! Flag the corresponding coarse cell as needing relimiting
@@ -312,27 +314,27 @@ recursive subroutine filrecur(level,nvar,valbig,aux,naux,t,mx,my, &
 
                     ! Determine slopes for interpolation
                     down_slope = (valcrse(ivalc(n,i_coarse,j_coarse)) - valcrse(ivalc(n,i_coarse-1,j_coarse)))
-                    up_slope = (valcrse(ivalc(n,i_coarse+1,j_coarse)) - valcrse(ivalc(n,i_coarse,j_coarse)))
+                    up_slope   = (valcrse(ivalc(n,i_coarse+1,j_coarse)) - valcrse(ivalc(n,i_coarse,j_coarse)))
                     if (up_slope * down_slope > 0.d0) then
-                        slope(1,i_coarse,j_coarse) = &
-                                        min(abs(up_slope), abs(down_slope)) * &
-                             sign(1.d0, valcrse(ivalc(n,i_coarse+1,j_coarse)) &
-                           - valcrse(ivalc(n,i_coarse-1,j_coarse)))
+                        slope(1,i_coarse,j_coarse) = min(abs(up_slope), abs(down_slope)) *    &
+                                          sign(1.d0, valcrse(ivalc(n,i_coarse+1,j_coarse))    &
+                                           - valcrse(ivalc(n,i_coarse-1,j_coarse)))
                     endif
 
                     down_slope = (valcrse(ivalc(n,i_coarse,j_coarse)) - valcrse(ivalc(n,i_coarse,j_coarse-1)))
-                    up_slope = (valcrse(ivalc(n,i_coarse,j_coarse+1)) - valcrse(ivalc(n,i_coarse,j_coarse)))
+                    up_slope   = (valcrse(ivalc(n,i_coarse,j_coarse+1)) - valcrse(ivalc(n,i_coarse,j_coarse)))
                     if (up_slope * down_slope > 0.d0) then
-                        slope(2,i_coarse,j_coarse) = &
-                                          min(abs(up_slope), abs(down_slope)) &
-                            * sign(1.d0, valcrse(ivalc(n,i_coarse,j_coarse+1)) &
-                                - valcrse(ivalc(n,i_coarse,j_coarse-1)))
+                        slope(2,i_coarse,j_coarse) = min(abs(up_slope), abs(down_slope))       &
+                                         * sign(1.d0, valcrse(ivalc(n,i_coarse,j_coarse+1))    &
+                                          - valcrse(ivalc(n,i_coarse,j_coarse-1)))
                     endif
 
                     ! Set initial values for max/min of current field
                     if (valcrse(ivalc(1,i_coarse,j_coarse)) > dry_tolerance) then
-                        vel_max(i_coarse,j_coarse) = valcrse(ivalc(n,i_coarse,j_coarse)) / valcrse(ivalc(1,i_coarse,j_coarse))
-                        vel_min(i_coarse,j_coarse) = valcrse(ivalc(n,i_coarse,j_coarse)) / valcrse(ivalc(1,i_coarse,j_coarse))
+                        vel_max(i_coarse,j_coarse) = valcrse(ivalc(n,i_coarse,j_coarse)) /    &
+                                                     valcrse(ivalc(1,i_coarse,j_coarse))
+                        vel_min(i_coarse,j_coarse) = valcrse(ivalc(n,i_coarse,j_coarse)) /    &
+                                                     valcrse(ivalc(1,i_coarse,j_coarse))
                     else
                         vel_min(i_coarse,j_coarse) = 0.d0
                         vel_max(i_coarse,j_coarse) = 0.d0
@@ -343,25 +345,21 @@ recursive subroutine filrecur(level,nvar,valbig,aux,naux,t,mx,my, &
                     ! but not interpolating depth linearly
                     do i =-1,1,2
                         if (valcrse(ivalc(1,i_coarse + i,j_coarse)) > dry_tolerance) then
-                            vel_max(i_coarse,j_coarse) = &
-                                max(vel_max(i_coarse,j_coarse), &
-                                    valcrse(ivalc(n,i_coarse + i,j_coarse)) &
-                                  / valcrse(ivalc(1,i_coarse + i,j_coarse)))
-                            vel_min(i_coarse,j_coarse) = &
-                                min(vel_min(i_coarse,j_coarse), &
-                                    valcrse(ivalc(n,i_coarse + i,j_coarse)) &
-                                  / valcrse(ivalc(1,i_coarse + i,j_coarse)))
+                            vel_max(i_coarse,j_coarse) =  max(vel_max(i_coarse,j_coarse),      &
+                                                valcrse(ivalc(n,i_coarse + i,j_coarse)) /      &
+                                                valcrse(ivalc(1,i_coarse + i,j_coarse)))
+                            vel_min(i_coarse,j_coarse) = min(vel_min(i_coarse,j_coarse),       &
+                                                valcrse(ivalc(n,i_coarse + i,j_coarse)) /      &
+                                                valcrse(ivalc(1,i_coarse + i,j_coarse)))
                         endif
 
                         if (valcrse(ivalc(1,i_coarse,j_coarse + i)) > dry_tolerance) then
-                            vel_max(i_coarse,j_coarse) = &
-                                max(vel_max(i_coarse,j_coarse), &
-                                      valcrse(ivalc(n,i_coarse,j_coarse + i)) &
-                                    / valcrse(ivalc(1,i_coarse,j_coarse + i)))
-                            vel_min(i_coarse,j_coarse) = &
-                                min(vel_min(i_coarse,j_coarse), &
-                                      valcrse(ivalc(n,i_coarse,j_coarse + i)) &
-                                    / valcrse(ivalc(1,i_coarse,j_coarse + i)))
+                            vel_max(i_coarse,j_coarse) = max(vel_max(i_coarse,j_coarse),       &
+                                                  valcrse(ivalc(n,i_coarse,j_coarse + i)) /    &
+                                                  valcrse(ivalc(1,i_coarse,j_coarse + i)))
+                            vel_min(i_coarse,j_coarse) = min(vel_min(i_coarse,j_coarse),       &
+                                                  valcrse(ivalc(n,i_coarse,j_coarse + i)) /    &
+                                                  valcrse(ivalc(1,i_coarse,j_coarse + i)))
 
                         endif
                     enddo
@@ -370,25 +368,27 @@ recursive subroutine filrecur(level,nvar,valbig,aux,naux,t,mx,my, &
 
             ! Determine momentum in fine cells
             do j_fine = 1, my_patch
-                j_coarse = 2 + (j_fine - (unset_indices(3) - jlo) - 1) / refinement_ratio_y
-                eta2 = (-0.5d0 + real(mod(j_fine - 1, refinement_ratio_y),kind=8)) / real(refinement_ratio_y,kind=8)
+                j_coarse     = floor((j_fine + jlo - 1) / ratio_y) - jplo + 1
+                ycent_coarse = ylow_coarse + (j_coarse-.5d0)*dy_coarse
+                ycent_fine   =  ylower + (j_fine-1+jlo + .5d0)*dy_fine
+                eta2         = (ycent_fine-ycent_coarse)/dy_coarse
+
                 do i_fine = 1, mx_patch
-                    i_coarse = 2 + (i_fine - (unset_indices(1) - ilo) - 1) / refinement_ratio_x
-                    eta1 = (-0.5d0 + real(mod(i_fine - 1, refinement_ratio_x),kind=8)) / real(refinement_ratio_x,kind=8)
-             
+                    i_coarse     = floor((i_fine+ilo-1) / ratio_x) - iplo + 1
+                    xcent_coarse = xlow_coarse + (i_coarse-.5d0)*dx_coarse
+                    xcent_fine   =  xlower + (i_fine-1+ilo + .5d0)*dx_fine
+                    eta1         = (xcent_fine-xcent_coarse)/dx_coarse
 
                     if (flaguse(i_fine,j_fine) == 0) then
                         ! Cell not already set
 
                         if (.not.(fine_flag(1,i_coarse,j_coarse))) then
                             ! This cell has no coarse cells that are dry
-                            hv_fine = valcrse(ivalc(n,i_coarse,j_coarse))  &
-                                            + eta1 * slope(1,i_coarse,j_coarse) &
+                            hv_fine = valcrse(ivalc(n,i_coarse,j_coarse))         &
+                                            + eta1 * slope(1,i_coarse,j_coarse)   &
                                             + eta2 * slope(2,i_coarse,j_coarse)
-                            v_fine = hv_fine  / valbig(1,i_fine + nrowst - 1, j_fine + ncolst - 1)
-                            if (v_fine < vel_min(i_coarse,j_coarse) .or. &
-                                v_fine > vel_max(i_coarse,j_coarse)) then
-
+                            v_fine = hv_fine  / valbig(1,i_fine+nrowst-1, j_fine+ncolst-1)
+                            if (v_fine<vel_min(i_coarse,j_coarse) .or.  v_fine>vel_max(i_coarse,j_coarse)) then
                                 fine_flag(n,i_coarse,j_coarse) = .true.
                                 reloop = .true.
                             else
@@ -403,13 +403,17 @@ recursive subroutine filrecur(level,nvar,valbig,aux,naux,t,mx,my, &
             ! gained momentum or if velocity bounds were violated
             if (reloop) then
               
-                do j_fine  = 1,my_patch
-                  j_coarse = 2 + (j_fine  - (unset_indices(3) - jlo) - 1) / refinement_ratio_y
-                  eta2 = (-0.5d0 + real(mod(j_fine - 1, refinement_ratio_y),kind=8)) / real(refinement_ratio_y, kind=8)
+                do j_fine  = 1, my_patch
+                  j_coarse     = floor((j_fine + jlo - 1) / ratio_y) - jplo + 1
+                  ycent_coarse = ylow_coarse + (j_coarse-.5d0)*dy_coarse
+                  ycent_fine   =  ylower + (j_fine-1+jlo + .5d0)*dy_fine
+                  eta2         = (ycent_fine-ycent_coarse)/dy_coarse
  
-                    do i_fine = 1,mx_patch
-                        i_coarse = 2 + (i_fine - (unset_indices(1) - ilo) - 1) / refinement_ratio_x
-                        eta1 = (-0.5d0 + real(mod(i_fine - 1, refinement_ratio_x),kind=8)) / real(refinement_ratio_x,kind=8)
+                    do i_fine = 1, mx_patch
+                       i_coarse     = floor((i_fine+ilo-1) / ratio_x) - iplo + 1
+                       xcent_coarse = xlow_coarse + (i_coarse-.5d0)*dx_coarse
+                       xcent_fine   =  xlower + (i_fine-1+ilo + .5d0)*dx_fine
+                       eta1         = (xcent_fine-xcent_coarse)/dx_coarse
 
                         if (flaguse(i_fine,j_fine) == 0) then
                             if (fine_flag(1,i_coarse,j_coarse) .or. fine_flag(n,i_coarse,j_coarse)) then
@@ -430,17 +434,17 @@ recursive subroutine filrecur(level,nvar,valbig,aux,naux,t,mx,my, &
                         endif
                     enddo
                 enddo
-            endif
-        enddo
+            endif ! end if reloop
+        enddo  ! end loop over nvar
     endif   ! end if patch not set
 
-    ! set bcs, whether or not recursive calls needed. set any part of patch that
-    ! stuck out
+    ! set bcs, whether or not recursive calls needed. set any part of patch that stuck out
     xhi_fine = xlower + (ihi + 1) * dx_fine
     yhi_fine = ylower + (jhi + 1) * dy_fine
-    call bc2amr(valbig,aux,mx,my,nvar,naux,dx_fine,dy_fine,level,t,xlow_fine, &
-               xhi_fine,ylow_fine,yhi_fine,xlower,ylower,xupper, &
-                yupper,xperdom,yperdom,spheredom)
+    if (patchOnly) then
+       call bc2amr(valbig,aux,mx,my,nvar,naux,dx_fine,dy_fine,level,t,   &
+                   xlow_fine,xhi_fine,ylow_fine,yhi_fine)
+    endif
 
 contains
 
