@@ -10,19 +10,21 @@ module multilayer_module
 
     implicit none
 
+    logical, private :: module_setup = .false.
+
     ! Storage parameters
     integer :: aux_layer_index
     
     ! Physical parameters
     integer :: num_layers
-    real(kind=8), allocatable :: rho(:), eta_init(:)
+    real(kind=8), allocatable :: eta_init(:)
     real(kind=8) :: r, one_minus_r
     
     ! Algorithm parameters
     integer :: eigen_method,inundation_method
     logical :: check_richardson
     real(kind=8) :: richardson_tolerance
-    real(kind=8), allocatable :: wave_tol(:), dry_tolerance(:)
+    real(kind=8), allocatable :: wave_tolerance(:), dry_tolerance(:)
     
     ! Output files
     integer, parameter :: KAPPA_UNIT = 42
@@ -48,100 +50,108 @@ contains
     ! ========================================================================
     subroutine set_multilayer(data_file)
 
-        use geoclaw_module, only: GEO_PARM_UNIT, geo_dry_tolerance => dry_tolerance
-        use storm_module, only: storm_type
+        use geoclaw_module, only: GEO_PARM_UNIT, rho
+        use geoclaw_module, only: geo_dry_tolerance => dry_tolerance
+        use storm_module, only: storm_specification_type
 
         implicit none
         character(len=*), optional, intent(in) :: data_file
         
         integer :: ios
         integer, parameter :: IOUNIT = 41
-        
-        ! Open file
-        if (present(data_file)) then
-            call opendatafile(IOUNIT, data_file)
-        else
-            call opendatafile(IOUNIT, 'multilayer.data')
-        endif
-        
-        write(GEO_PARM_UNIT,*) ' '
-        write(GEO_PARM_UNIT,*) '--------------------------------------------'
-        write(GEO_PARM_UNIT,*) 'Multilayer Parameters:'
-        write(GEO_PARM_UNIT,*) '----------------------'
+            
 
-        ! Read in parameters
-        read(IOUNIT,"(i3)") num_layers
-        allocate(rho(num_layers))
-        allocate(eta_init(num_layers))
-        allocate(wave_tol(num_layers))
-        allocate(dry_tolerance(num_layers))
+        if (.not.module_setup) then
+            ! Open file
+            if (present(data_file)) then
+                call opendatafile(IOUNIT, data_file)
+            else
+                call opendatafile(IOUNIT, 'multilayer.data')
+            endif
+            
+            write(GEO_PARM_UNIT,*) ' '
+            write(GEO_PARM_UNIT,*) '--------------------------------------------'
+            write(GEO_PARM_UNIT,*) 'Multilayer Parameters:'
+            write(GEO_PARM_UNIT,*) '----------------------'
 
-        read(IOUNIT,*) rho
-        if (num_layers > 1) then
-            r = rho(1) / rho(2)
-            one_minus_r = 1.d0 - r
-        else
-            r = -1.d0
-            one_minus_r = 0.d0
-        endif
-        read(IOUNIT, *) eta_init
-        read(IOUNIT, *)
+            ! Read in parameters
+            read(IOUNIT,"(i3)") num_layers
+            allocate(eta_init(num_layers))
+            allocate(wave_tolerance(num_layers))
+            allocate(dry_tolerance(num_layers))
 
-        ! Algorithmic parameters
-        read(IOUNIT, *) check_richardson
-        read(IOUNIT, "(d16.8)") richardson_tolerance
-        read(IOUNIT, "(i1)") eigen_method
-        read(IOUNIT, "(i1)") inundation_method
-        
-        close(IOUNIT) 
+            ! read(IOUNIT,*) rho
+            ! The densities are in the geoclaw_module, check here to make surge
+            ! there are enough values
+            if (size(rho, 1) /= num_layers) then
+                print *, "The number of values of the water density " //       &
+                          "(", size(rho, 1),") does not match the number " //  &
+                          "of layers (", num_layers,")."
+                stop
+            end if
+            if (num_layers > 1) then
+                r = rho(1) / rho(2)
+                one_minus_r = 1.d0 - r
+            else
+                r = -1.d0
+                one_minus_r = 0.d0
+            endif
+            read(IOUNIT, *) eta_init
+            read(IOUNIT, *) wave_tolerance
+            read(IOUNIT, *) aux_layer_index
+            read(IOUNIT, *)
 
-        ! Set layer index - depends on whether a storm surge is being modeled
-        if (storm_type == 0) then
-            aux_layer_index = 5
-        else
-            aux_layer_index = 8
+            ! Algorithmic parameters
+            read(IOUNIT, *) check_richardson
+            read(IOUNIT, "(d16.8)") richardson_tolerance
+            read(IOUNIT, "(i1)") eigen_method
+            read(IOUNIT, "(i1)") inundation_method
+            
+            close(IOUNIT) 
+
+            ! Currently just set dry_tolerance(:) = dry_tolerance
+            dry_tolerance = geo_dry_tolerance
+
+            ! Set eigen functions
+            select case(eigen_method)
+                case(1:2)
+                    eigen_func => linearized_eigen
+                case(3)
+                    eigen_func => vel_diff_eigen
+                case(4)
+                    eigen_func => lapack_eigen
+                case default
+                    print *, "Invalid eigenspace method requested: ", eigen_method
+                    stop
+            end select
+            select case(inundation_method)
+                case(1:2)
+                    inundation_eigen_func => linearized_eigen
+                case(3)
+                    inundation_eigen_func => vel_diff_eigen
+                case(4:5)
+                    inundation_eigen_func => lapack_eigen
+                case default
+                    print *, "Invalid eigenspace method requested: ", eigen_method
+                    stop
+            end select
+
+            ! Open Kappa output file if num_layers > 1
+            ! Open file for writing hyperbolicity warnings if multiple layers
+            if (num_layers > 1 .and. check_richardson) then
+                open(unit=KAPPA_UNIT, file='fort.kappa', iostat=ios, &
+                        status="unknown", action="write")
+                if ( ios /= 0 ) stop "Error opening file name fort.kappa"
+            endif
+
+            write(GEO_PARM_UNIT,*) '   check_richardson:',check_richardson
+            write(GEO_PARM_UNIT,*) '   richardson_tolerance:',richardson_tolerance
+            write(GEO_PARM_UNIT,*) '   eigen_method:',eigen_method
+            write(GEO_PARM_UNIT,*) '   inundation_method:',inundation_method
+            write(GEO_PARM_UNIT,*) '   dry_tolerance:',dry_tolerance
+
+            module_setup = .true.
         end if
-
-        ! Currently just set dry_tolerance(:) = dry_tolerance
-        dry_tolerance = geo_dry_tolerance
-
-        ! Set eigen functions
-        select case(eigen_method)
-            case(1:2)
-                eigen_func => linearized_eigen
-            case(3)
-                eigen_func => vel_diff_eigen
-            case(4)
-                eigen_func => lapack_eigen
-            case default
-                print *, "Invalid eigenspace method requested: ", eigen_method
-                stop
-        end select
-        select case(inundation_method)
-            case(1:2)
-                inundation_eigen_func => linearized_eigen
-            case(3)
-                inundation_eigen_func => vel_diff_eigen
-            case(4:5)
-                inundation_eigen_func => lapack_eigen
-            case default
-                print *, "Invalid eigenspace method requested: ", eigen_method
-                stop
-        end select
-
-        ! Open Kappa output file if num_layers > 1
-        ! Open file for writing hyperbolicity warnings if multiple layers
-        if (num_layers > 1 .and. check_richardson) then
-            open(unit=KAPPA_UNIT, file='fort.kappa', iostat=ios, &
-                    status="unknown", action="write")
-            if ( ios /= 0 ) stop "Error opening file name fort.kappa"
-        endif
-
-        write(GEO_PARM_UNIT,*) '   check_richardson:',check_richardson
-        write(GEO_PARM_UNIT,*) '   richardson_tolerance:',richardson_tolerance
-        write(GEO_PARM_UNIT,*) '   eigen_method:',eigen_method
-        write(GEO_PARM_UNIT,*) '   inundation_method:',inundation_method
-        write(GEO_PARM_UNIT,*) '   dry_tolerance:',dry_tolerance
         
     end subroutine set_multilayer
 
@@ -241,7 +251,7 @@ contains
         
     end subroutine vel_diff_eigen
 
-
+#ifdef LAPACK_AVAIL
     subroutine lapack_eigen(h_l,h_r,u_l,u_r,v_l,v_r,n_index,t_index,s,eig_vec)
 
         use geoclaw_module, only: grav
@@ -316,7 +326,26 @@ contains
         enddo
 
     end subroutine lapack_eigen
+#else
 
+    subroutine lapack_eigen(h_l,h_r,u_l,u_r,v_l,v_r,n_index,t_index,s,eig_vec)
+
+        use geoclaw_module, only: grav
+
+        implicit none
+        
+        ! Input
+        double precision, dimension(2), intent(in) :: h_l,h_r,u_l,u_r,v_l,v_r
+        integer, intent(in) :: n_index,t_index
+        
+        ! Output
+        double precision, intent(inout) :: s(6),eig_vec(6,6)
+  
+        stop "LAPACK eigensolver was not available at build time."
+
+    end subroutine lapack_eigen
+
+#endif
 
     ! ==========================================================================
     !  Single layer eigensolver
