@@ -2,7 +2,7 @@ c
 c -------------------------------------------------------------
 c
       subroutine stepgrid(q,fm,fp,gm,gp,mitot,mjtot,mbc,dt,dtnew,dx,dy,
-     &                  nvar,xlow,ylow,time,mptr,maux,aux)
+     &                  nvar,xlow,ylow,time,mptr,maux,aux,actualstep)
 c
 c
 c ::::::::::::::::::: STEPGRID ::::::::::::::::::::::::::::::::::::
@@ -45,8 +45,9 @@ c      dimension work(mwork)
 
       logical :: debug = .false.
       logical :: dump = .false.
+      logical, intent (in) :: actualstep
 c
-      
+      tcfmax = -rinfinity
       level = node(nestlevel,mptr)
 
       if (dump) then
@@ -153,7 +154,7 @@ c               # test if arrival times should be output
 c::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 
        call b4step2(mbc,mx,my,nvar,q,
-     &             xlowmbc,ylowmbc,dx,dy,time,dt,maux,aux)
+     &             xlowmbc,ylowmbc,dx,dy,time,dt,maux,aux,actualstep)
       
 c::::::::::::::::::::::::FIXED GRID DATA before step:::::::::::::::::::::::
 c     # fill in values at fixed grid points effected at time tc0
