@@ -9,12 +9,20 @@ c
       use amr_module
       use topo_module, only: dt_max_dtopo, num_dtopo, topo_finalized,
      &                       aux_finalized, topo0work
+      use gauges_module, only: setbestsrc, num_gauges
+      use gauges_module, only: print_gauges_and_reset_nextLoc
+
+      use storm_module, only: landfall, display_landfall_time
+
 
       implicit double precision (a-h,o-z)
 
       logical vtime,dumpout/.false./,dumpchk/.false./,rest,dump_final
       dimension dtnew(maxlv), ntogo(maxlv), tlevel(maxlv)
-      integer clock_start, clock_finish, clock_rate
+      integer(kind=8) :: clock_start, clock_finish, clock_rate
+      integer(kind=8) :: tick_clock_finish, tick_clock_rate
+      character(len=128) :: time_format
+      real(kind=8) cpu_start,cpu_finish
 
 c
 c :::::::::::::::::::::::::::: TICK :::::::::::::::::::::::::::::
@@ -44,6 +52,7 @@ c          have its error estimated and finer levels should be regridded.
 c ::::::::::::::::::::::::::::::::::::;::::::::::::::::::::::::::
 c
 
+
       ncycle         = nstart
       call setbestsrc()     ! need at very start of run, including restart
       if (iout .eq. 0) then
@@ -66,7 +75,7 @@ c              # restart: make sure output times start after restart time
       endif
 
       nextchk = 1
-      if ((nstart .gt. 0) .and. (checkpt_style.eq.2)) then
+      if ((nstart .gt. 0) .and. (abs(checkpt_style).eq.2)) then
 c        if this is a restart, make sure chkpt times start after restart time
          do ii = 1, nchkpt
            if (tchk(ii) .gt. time) then
@@ -120,12 +129,12 @@ c        write(*,*)" old possk is ", possk(1)
          if (.false.) then  
             write(*,122) diffdt,outtime  ! notify of change
  122        format(" Adjusting timestep by ",e10.3,
-     .             " to hit output time of ",e12.6)
+     .             " to hit output time of ",e13.6)
 c           write(*,*)" new possk is ", possk(1)
             if (diffdt .lt. 0.) then ! new step is slightly larger
               pctIncrease = -100.*diffdt/oldposs   ! minus sign to make whole expr. positive
               write(*,123) pctIncrease
- 123          format(" New step is ",e8.2," % larger.",
+ 123          format(" New step is ",e9.2," % larger.",
      .               "  Should still be stable")
               endif
             endif
@@ -155,9 +164,10 @@ c        ## adjust time step  to hit chktime exactly, and do checkpointing
 c
       level        = 1
       ntogo(level) = 1
-      do i = 1, maxlv
-         dtnew(i)  = rinfinity
-      enddo
+      dtnew(1:maxlv) = rinfinity
+C       do i = 1, maxlv
+C          dtnew(i)  = rinfinity
+C       enddo
 
 c     We should take at least one step on all levels after any
 c     moving topography (dtopo) has been finalized to insure that
@@ -165,7 +175,7 @@ c     all aux arrays are consistent with the final topography.
 c     The variable aux_finalized is incremented so that we can check
 c     if this is true by checking if aux_finalized == 2 elsewhere in code.
 
-	  if (aux_finalized .eq. 1 .and. num_dtopo > 0) then
+      if (aux_finalized .eq. 1 .and. num_dtopo > 0) then
 c         # this is only true once, and only if there was moving topo
           deallocate(topo0work)
           endif 
@@ -208,9 +218,12 @@ c
 101       format(8h  level ,i5,32h  stays fixed during regridding )
 
           call system_clock(clock_start,clock_rate)
+          call cpu_time(cpu_start)
           call regrid(nvar,lbase,cut,naux,start_time)
           call system_clock(clock_finish,clock_rate)
+          call cpu_time(cpu_finish)
           timeRegridding = timeRegridding + clock_finish - clock_start
+          timeRegriddingCPU=timeRegriddingCPU+cpu_finish-cpu_start
 
           call setbestsrc()     ! need at every grid change
 c         call conck(1,nvar,naux,time,rest)
@@ -249,19 +262,23 @@ c
 
           call advanc(level,nvar,dtlevnew,vtime,naux)
 
-c         # rjl modified 6/17/05 to print out *after* advanc and print cfl
-c         # rjl & mjb changed to cfl_level, 3/17/10
-
+c Output time info
           timenew = tlevel(level)+possk(level)
+          time_format = "(' AMRCLAW: level ',i2,'  CFL = ',e8.3," //
+     &                  "'  dt = ',e10.4,  '  final t = ',e12.6)"
+          if (display_landfall_time) then
+c           Convert time to days            
+            timenew = timenew / (3.6d3 * 24d0)
+            time_format = "(' AMRCLAW: level ',i2,'  CFL = ',e8.3," //
+     &                  "'  dt = ',e10.4,  '  final t = ', f5.2)"
+          end if
           if (tprint) then
-              write(outunit,100)level,cfl_level,possk(level),timenew
-              endif
+              write(outunit, time_format) level, cfl_level, 
+     &                                    possk(level), timenew
+          endif
           if (method(4).ge.level) then
-              write(6,100)level,cfl_level,possk(level),timenew
-              endif
-100       format(' AMRCLAW: level ',i2,'  CFL = ',e8.3,
-     &           '  dt = ',e10.4,  '  final t = ',e12.6)
-
+              print time_format, level, cfl_level, possk(level), timenew
+          endif
 
 c        # to debug individual grid updates...
 c        call valout(level,level,time,nvar,naux)
@@ -318,6 +335,11 @@ c                   adjust time steps for this and finer levels
      &                             kratio(level-1),level
                      write(6,*) "Writing checkpoint file at t = ",time
                      call check(ncycle,time,nvar,naux)
+                     if (num_gauges .gt. 0) then
+                        do ii = 1, num_gauges
+                           call print_gauges_and_reset_nextLoc(ii)
+                        end do
+                     endif
                      stop
                  endif
 
@@ -340,16 +362,6 @@ c
           ncycle  = ncycle + 1
           call conck(1,nvar,naux,time,rest)
 
-       if ((checkpt_style.eq.3 .and. 
-     &      mod(ncycle,checkpt_interval).eq.0) .or. dumpchk) then
-                call check(ncycle,time,nvar,naux)
-                dumpchk = .true.
-       endif
-
-       if ((mod(ncycle,iout).eq.0) .or. dumpout) then
-         call valout(1,lfine,time,nvar,naux)
-         if (printout) call outtre(mstart,.true.,nvar,naux)
-       endif
 
       if ( .not.vtime) goto 201
 
@@ -385,7 +397,28 @@ c             ! use same alg. as when setting refinement when first make new fin
 
       endif
 
- 201  go to 20
+ 201  if ((abs(checkpt_style).eq.3 .and. 
+     &      mod(ncycle,checkpt_interval).eq.0) .or. dumpchk) then
+                call check(ncycle,time,nvar,naux)
+                dumpchk = .true.
+               if (num_gauges .gt. 0) then
+                  do ii = 1, num_gauges
+                     call print_gauges_and_reset_nextLoc(ii)
+                  end do
+               endif
+       endif
+
+       if ((mod(ncycle,iout).eq.0) .or. dumpout) then
+         call valout(1,lfine,time,nvar,naux)
+         if (printout) call outtre(mstart,.true.,nvar,naux)
+         if (num_gauges .gt. 0) then
+            do ii = 1, num_gauges
+               call print_gauges_and_reset_nextLoc(ii)
+            end do
+         endif
+       endif
+
+      go to 20
 c
 999   continue
 
@@ -412,15 +445,34 @@ c
       if (dump_final) then
            call valout(1,lfine,time,nvar,naux)
            if (printout) call outtre(mstart,.true.,nvar,naux)
+           if (num_gauges .gt. 0) then
+              do ii = 1, num_gauges
+                 call print_gauges_and_reset_nextLoc(ii)
+              end do
+           endif
       endif
 
 c  # checkpoint everything for possible future restart
 c  # (unless we just did it based on dumpchk)
 c
+      call system_clock(tick_clock_finish,tick_clock_rate)
+      call cpu_time(tick_cpu_finish)
+      timeTick = timeTick + tick_clock_finish - tick_clock_start 
+      timeTickCPU = timeTickCPU + tick_cpu_finish - tick_cpu_start 
 
-      if ((checkpt_style .gt. 0) .and. (.not. dumpchk)) then
-           call check(ncycle,time,nvar,naux)
-         endif
+
+c  # checkpoint everything for possible future restart
+c  # (unless we just did it based on dumpchk)
+c
+      if (checkpt_style .ne. 0) then  ! want a chckpt
+         ! check if just did it so dont do it twice
+         if (.not. dumpchk) call check(ncycle,time,nvar,naux)
+      endif
+      if (num_gauges .gt. 0) then
+         do ii = 1, num_gauges
+            call print_gauges_and_reset_nextLoc(ii)
+         end do
+      endif
 
       write(6,*) "Done integrating to time ",time
       return
