@@ -12,9 +12,12 @@ for example, to get colors ranging from white to green.
 See matplotlib._cm for the data defining various maps.
 """
 
+from __future__ import absolute_import
+from __future__ import print_function
 import numpy
 import matplotlib.colors as colors
 import matplotlib.pyplot as plt
+from six.moves import range
 
 #-------------------------
 def make_colormap(color_list):
@@ -27,102 +30,126 @@ def make_colormap(color_list):
     The z values (dictionary keys) are real numbers and the values
     colors[z] can be either an RGB list, e.g. [1,0,0] for red, or an
     html hex string, e.g. "#ff0000" for red.
+
+    Optionally, an alpha channel can be added to indicate levels of transparency.
+    In this case, colors should have a fourth argument, a value between 0 and 1,
+    where zero is transparent and 1 is opaque. Ex.
+
+              blue_semi_transparent = [0.0, 0.0, 1.0, 0.5]
+
     """
 
-    
-    
-    z = numpy.sort(color_list.keys())
+
+
+    z = numpy.sort(list(color_list.keys()))
     n = len(z)
     z1 = min(z)
     zn = max(z)
     x0 = (z - z1) / (zn - z1)
-    
+
     CC = colors.ColorConverter()
     R = []
     G = []
     B = []
+    A = []
     for i in range(n):
         #i'th color at level z[i]:
-        Ci = color_list[z[i]]      
+        Ci = color_list[z[i]]
         if type(Ci) == str:
             # a hex string of form '#ff0000' for example (for red)
-            RGB = CC.to_rgb(Ci)
+            RGBA = CC.to_rgba(Ci,1.0)
         else:
-            # assume it's an RGB triple already:
-            RGB = Ci
-        R.append(RGB[0])
-        G.append(RGB[1])
-        B.append(RGB[2])
+            # assume it's an RGB (or RGBA) tuple already:
+            if (len(Ci) == 3):
+                Ci = numpy.concatenate((Ci,[1.0]))  # Add alpha channel
+            RGBA = Ci
+
+        R.append(RGBA[0])
+        G.append(RGBA[1])
+        B.append(RGBA[2])
+        A.append(RGBA[3])
 
     cmap_dict = {}
     cmap_dict['red'] = [(x0[i],R[i],R[i]) for i in range(len(R))]
     cmap_dict['green'] = [(x0[i],G[i],G[i]) for i in range(len(G))]
     cmap_dict['blue'] = [(x0[i],B[i],B[i]) for i in range(len(B))]
+    cmap_dict['alpha'] = [(x0[i],A[i],A[i]) for i in range(len(A))]
     mymap = colors.LinearSegmentedColormap('mymap',cmap_dict)
     return mymap
 
 
-def add_colormaps(colormaps, data_limits=[0.0,1.0], data_break=0.5, colormap_name="something"):
-    r"""Concatenate colormaps in *colormaps* list.
+def add_colormaps(colormaps, data_limits=[0.0,1.0], data_break=0.5, 
+                             colormap_name="JohnDoe"):
+    r"""Concatenate colormaps in *colormaps* list and return Normalize object.
 
     Appends the colormaps in the list *colormaps* adjusting the mapping to the
     colormap such that it maps the data space limits *data_limits* and puts the
-    break in the colormaps at *data_break* which is again in data space.  The 
-    argument *colormap_name* labels the colormap and is passed to the init 
+    break in the colormaps at *data_break* which is again in data space.  The
+    argument *colormap_name* labels the colormap and is passed to the init
     method of `matplotlib.colors.LinearSegmentedColormap`.
 
     Originally contributed by Damon McDougall
 
-    returns `matplotlib.colors.LinearSegmentedColormap`
+    returns `matplotlib.colors.LinearSegmentedColormap`, 
+            `matplotlib.colors.Normalize`
 
     Example
     -------
-    This example takes two colormaps whose data ranges from [-1.0, 5.0] where 
+    This example takes two colormaps whose data ranges from [-1.0, 5.0] where
     the break in the colormaps occurs at 1.0.
 
     >>> import matplotlib.pyplot as plt
     >>> import clawpack.visclaw.colormaps as colormaps
     >>> import numpy
     >>> cmaps = (plt.get_cmap("BuGn_r"), plt.get_cmap("Blues_r"))
-    >>> new_cmap = colormaps.add_colormaps(cmaps, data_limits=[-1.0, 5.0], 
+    >>> new_cmap, norm = colormaps.add_colormaps(cmaps, data_limits=[-1.0, 5.0],
         data_break=1.0)
     >>> x = numpy.linspace(-1,1,100)
     >>> y = x
     >>> X, Y = numpy.meshgrid(x, y)
     >>> fig = plt.figure()
     >>> axes = fig.add_subplot(1,1,1)
-    >>> plot = axes.pcolor(X, Y, 3.0 * X + 2.0, cmap=new_cmap)
+    >>> plot = axes.pcolor(X, Y, 3.0 * X + 2.0, cmap=new_cmap, norm=norm)
     >>> fig.colorbar(plot)
     >>> plt.show()
 
     """
 
-    break_location = (data_break - data_limits[0])      \
-                            / (data_limits[1] - data_limits[0])
-    
     lhs_dict = colormaps[0]._segmentdata
     rhs_dict = colormaps[1]._segmentdata
-    new_dict = dict(red=[], green=[], blue=[])
+    new_dict = dict(red=[], green=[], blue=[], alpha=[])
 
-    # Scale rhs by half
+    # Add first colorbar
     for key in rhs_dict:
         val_list = rhs_dict[key]
-        print(key, val_list)
         for val in val_list:
-            new_dict[key].append((val[0] * break_location, val[1], val[2]))
+            new_dict[key].append((val[0] * 0.5, val[1], val[2]))
 
-    # Append lhs
+    if 'alpha' not in list(rhs_dict.keys()):
+        new_dict['alpha'].append((0.0,1.0,1.0))
+
+    # Add second colorbar
     for key in lhs_dict:
         val_list = lhs_dict[key]
-        print(key, val_list)
         for val in val_list:
-            new_dict[key].append((break_location + val[0] * \
-                                    (1.0 - break_location), val[1], val[2]))
+            new_dict[key].append(((val[0] + 1.0) * 0.5, val[1], val[2]))
+
+    if 'alpha' not in list(lhs_dict.keys()):
+        new_dict['alpha'].append((1.0,1.0,1.0))
 
     N = 256
     gamma = 1.0
 
-    return colors.LinearSegmentedColormap(colormap_name, new_dict, N, gamma)
+    cmap = colors.LinearSegmentedColormap(colormap_name, new_dict, N, gamma)
+
+    # Compute new norm object
+    bounds = numpy.empty(N)
+    bounds[:int(N / 2)] = numpy.linspace(data_limits[0], data_break, int(N / 2))
+    bounds[int(N / 2):] = numpy.linspace(data_break, data_limits[1], 
+                                                             int(N / 2) + N % 2)
+    norm = colors.BoundaryNorm(boundaries=bounds, ncolors=N)
+
+    return cmap, norm
 
 
 def showcolors(cmap):
@@ -182,13 +209,13 @@ schlieren_greens = schlieren_colormap('g')
 def make_amrcolors(nlevels=4):
 #-------------------------------
     """
-    Make lists of colors useful for distinguishing different patches when 
+    Make lists of colors useful for distinguishing different patches when
     plotting AMR results.
 
     INPUT::
        nlevels: maximum number of AMR levels expected.
     OUTPUT::
-       (linecolors, bgcolors) 
+       (linecolors, bgcolors)
        linecolors = list of nlevels colors for patch lines, contour lines
        bgcolors = list of nlevels pale colors for patch background
     """
@@ -207,6 +234,6 @@ def make_amrcolors(nlevels=4):
         linecolors = linecolors[:nlevels]
         bgcolors = bgcolors[:nlevels]
     else:
-        print "*** Warning, suggest nlevels <= 16"
+        print("*** Warning, suggest nlevels <= 16")
 
     return (linecolors, bgcolors)
