@@ -2,30 +2,34 @@
 Tools for plotting data from gauges, gauge locations, etc.
 """
 
+from __future__ import absolute_import
+from __future__ import print_function
 import os,sys,shutil,glob
 import string,re
 import time
 import traceback
+import warnings
 
 import numpy as np
 
 import clawpack.clawutil.data as clawdata
 
 from clawpack.visclaw.frametools import set_show
+from six.moves import range
 
 plotter = 'matplotlib'
 if plotter == 'matplotlib':
-    if not sys.modules.has_key('matplotlib'):
+    if 'matplotlib' not in sys.modules:
         try:
             import matplotlib
             matplotlib.use('Agg')  # Use an image backend
         except:
-            print "*** Error: problem importing matplotlib"
+            print("*** Error: problem importing matplotlib")
 
 try:
     import pylab
 except:
-    print "*** Error: problem importing pylab"
+    print("*** Error: problem importing pylab")
 
 # Gauge solution class
 class GaugeSolution(object):
@@ -81,8 +85,11 @@ class GaugeSolution(object):
         return locals()
     location = property(**location())
 
-    def __init__(self,number,location=None):
+    def __init__(self, number, location=None):
         
+        warnings.warn("This version of GaugeSolution is deprecated, use the ",
+                      "class definition in clawpack.amrclaw.gauges instead.")
+
         # Gauge descriptors
         self.number = number
         self._location = None
@@ -100,8 +107,8 @@ class GaugeSolution(object):
             output = None
         else:
             output = "%4i" % self.number
-            output = " ".join((output,"%19.10e" % self.location[0]))
-            output = " ".join((output,"%17.10e" % self.location[1]))
+            for j in range(len(self.location)):
+                output = " ".join((output,"%17.10e" % self.location[j]))
             output = " ".join((output,"%13.6e" % self.t1))
             output = " ".join((output,"%13.6e\n" % self.t2))
         return output
@@ -122,11 +129,10 @@ def plotgauge(gaugeno, plotdata, verbose=False):
 
     """
 
-
     if verbose:  
         gaugesoln = plotdata.getgauge(gaugeno)
-        print '    Plotting gauge %s  at x = %s, y = %s ... '  \
-                 % (gaugeno, gaugesoln.location[0], gaugesoln.location[1])
+        print('    Plotting gauge %s  at x = %s, y = %s ... '  \
+                 % (gaugeno, gaugesoln.location[0], gaugesoln.location[1]))
 
     if plotdata.mode() == 'iplotclaw':
         pylab.ion()
@@ -135,13 +141,13 @@ def plotgauge(gaugeno, plotdata, verbose=False):
     try:
         plotfigure_dict = plotdata.plotfigure_dict
     except:
-        print '*** Error in plotgauge: plotdata missing plotfigure_dict'
-        print '*** This should not happen'
+        print('*** Error in plotgauge: plotdata missing plotfigure_dict')
+        print('*** This should not happen')
         return None
 
     if len(plotfigure_dict) == 0:
-        print '*** Warning in plotgauge: plotdata has empty plotfigure_dict'
-        print '*** Apparently no figures to plot'
+        print('*** Warning in plotgauge: plotdata has empty plotfigure_dict')
+        print('*** Apparently no figures to plot')
 
 
 
@@ -169,7 +175,7 @@ def plotgauge(gaugeno, plotdata, verbose=False):
                 output = beforegauge(current_data)
                 if output: current_data = output
             except:
-                print '*** Error in beforegauge ***'
+                print('*** Error in beforegauge ***')
                 raise
 
 
@@ -179,8 +185,13 @@ def plotgauge(gaugeno, plotdata, verbose=False):
  
     if plotdata._mode == 'iplotclaw':
         gaugesoln = plotdata.getgauge(gaugeno)
-        print '    Plotting Gauge %s  at x = %s, y = %s ... '  \
-                 % (gaugeno, gaugesoln.location[0], gaugesoln.location[1])
+        gaugeloc = gaugesoln.location
+        if len(gaugeloc) == 2:
+            print('    Plotting Gauge %s  at x = %s, y = %s ... '  \
+                     % (gaugeno, gaugeloc[0], gaugeloc[1]))
+        elif len(gaugeloc) == 3:
+            print('    Plotting Gauge %s  at x = %s, y = %s, z = %s ... '  \
+                     % (gaugeno, gaugeloc[0], gaugeloc[1], gaugeloc[2]))
         requested_fignos = plotdata.iplotclaw_fignos
     else:
         requested_fignos = plotdata.print_fignos
@@ -205,7 +216,7 @@ def plotgauge(gaugeno, plotdata, verbose=False):
         plotted_fignos.append(figno)
 
 
-        if not plotfigure.kwargs.has_key('facecolor'):
+        if 'facecolor' not in plotfigure.kwargs:
             # use Clawpack's default bg color (tan)
             plotfigure.kwargs['facecolor'] = '#ffeebb'   
 
@@ -219,13 +230,13 @@ def plotgauge(gaugeno, plotdata, verbose=False):
         try:
             plotaxes_dict = plotfigure.plotaxes_dict
         except:
-            print '*** Error in plotgauge: plotdata missing plotaxes_dict'
-            print '*** This should not happen'
+            print('*** Error in plotgauge: plotdata missing plotaxes_dict')
+            print('*** This should not happen')
             return  None
 
         if (len(plotaxes_dict) == 0) or (len(plotfigure._axesnames) == 0):
-            print '*** Warning in plotgauge: plotdata has empty plotaxes_dict'
-            print '*** Apparently no axes to plot in figno ',figno
+            print('*** Warning in plotgauge: plotdata has empty plotaxes_dict')
+            print('*** Apparently no axes to plot in figno ',figno)
 
         # loop over axes to appear on this figure:
         # ----------------------------------------
@@ -239,9 +250,6 @@ def plotgauge(gaugeno, plotdata, verbose=False):
             axescmd = getattr(plotaxes,'axescmd','subplot(1,1,1)')
             axescmd = 'plotaxes._handle = pylab.%s' % axescmd
             exec(axescmd)
-            pylab.hold(True)
-
-
 
             # loop over items:
             # ----------------
@@ -263,53 +271,53 @@ def plotgauge(gaugeno, plotdata, verbose=False):
                         output = plotgauge1(gaugesoln,plotitem,current_data)
                         if output: current_data = output
                         if verbose:  
-                                print '      Plotted  plotitem ', itemname
+                                print('      Plotted  plotitem ', itemname)
                     except:
-                        print '*** Error in plotgauge: problem calling plotgauge1'
+                        print('*** Error in plotgauge: problem calling plotgauge1')
                         traceback.print_exc()
                         return None
 
             # end of loop over plotitems
 
-
-        for itemname in plotaxes._itemnames:
-            plotitem = plotaxes.plotitem_dict[itemname]
-
-        pylab.title("%s at gauge %s" % (plotaxes.title,gaugeno))
-
-
-        # call an afteraxes function if present:
-        afteraxes =  getattr(plotaxes, 'afteraxes', None)
-        if afteraxes:
-            if isinstance(afteraxes, str):
-                # a string to be executed
-                exec(afteraxes)
-            else:
-                # assume it's a function
+    
+            pylab.title("%s at gauge %s" % (plotaxes.title,gaugeno))
+    
+            if plotaxes.time_label is not None:
+                pylab.xlabel(plotaxes.time_label, **plotaxes.time_label_kwargs)
+    
+    
+            # call an afteraxes function if present:
+            afteraxes =  getattr(plotaxes, 'afteraxes', None)
+            if afteraxes:
+                if isinstance(afteraxes, str):
+                    # a string to be executed
+                    exec(afteraxes)
+                else:
+                    # assume it's a function
+                    try:
+                        current_data.add_attribute("plotaxes",plotaxes)
+                        current_data.add_attribute("plotfigure",plotaxes._plotfigure)
+                        output = afteraxes(current_data)
+                        if output: current_data = output
+                    except:
+                        print('*** Error in afteraxes ***')
+                        raise
+    
+            if plotaxes.scaled:
+                pylab.axis('scaled')
+    
+            # set axes limits:
+            if (plotaxes.xlimits is not None) & (type(plotaxes.xlimits) is not str):
                 try:
-                    current_data.add_attribute("plotaxes",plotaxes)
-                    current_data.add_attribute("plotfigure",plotaxes._plotfigure)
-                    output = afteraxes(current_data)
-                    if output: current_data = output
+                    pylab.xlim(plotaxes.xlimits[0], plotaxes.xlimits[1])
                 except:
-                    print '*** Error in afteraxes ***'
-                    raise
-
-        if plotaxes.scaled:
-            pylab.axis('scaled')
-
-        # set axes limits:
-        if (plotaxes.xlimits is not None) & (type(plotaxes.xlimits) is not str):
-            try:
-                pylab.xlim(plotaxes.xlimits[0], plotaxes.xlimits[1])
-            except:
-                pass  # let axis be set automatically
-        if (plotaxes.ylimits is not None) & (type(plotaxes.ylimits) is not str):
-            try:
-                pylab.ylim(plotaxes.ylimits[0], plotaxes.ylimits[1])
-            except:
-                pass  # let axis be set automatically
-
+                    pass  # let axis be set automatically
+            if (plotaxes.ylimits is not None) & (type(plotaxes.ylimits) is not str):
+                try:
+                    pylab.ylim(plotaxes.ylimits[0], plotaxes.ylimits[1])
+                except:
+                    pass  # let axis be set automatically
+    
 
             # end of loop over plotaxes
             
@@ -328,7 +336,7 @@ def plotgauge(gaugeno, plotdata, verbose=False):
                 output = aftergauge(current_data)
                 if output: current_data = output
             except:
-                print '*** Error in aftergauge ***'
+                print('*** Error in aftergauge ***')
                 raise
 
 
@@ -339,7 +347,7 @@ def plotgauge(gaugeno, plotdata, verbose=False):
         pylab.draw()
 
     if verbose:
-        print '    Done with plotgauge for gauge %i' % (gaugeno)
+        print('    Done with plotgauge for gauge %i' % (gaugeno))
 
     
     # print the figure(s) to file(s) if requested:
@@ -370,6 +378,11 @@ def plotgauge1(gaugesoln, plotitem, current_data):
 
     """
 
+    if not gaugesoln.is_valid():
+        import warnings
+        warnings.warn("Gauge has not been initialized properly.")
+        return
+
     plotdata = plotitem._plotdata
     plotfigure = plotitem._plotfigure
     plotaxes = plotitem._plotaxes
@@ -391,7 +404,8 @@ def plotgauge1(gaugesoln, plotitem, current_data):
     color = plotitem.color
     plotstyle = plotitem.plotstyle
 
-    t = gaugesoln.t
+    t = gaugesoln.t * plotaxes.time_scale
+
     if type(plot_var) is int:
         var = gaugesoln.q[plot_var,:]
     else:
@@ -402,24 +416,12 @@ def plotgauge1(gaugesoln, plotitem, current_data):
     tmax = t.max()
     varmax = var.max()
 
-    # The plot commands using matplotlib:
-
-    pylab.hold(True)
-
-    # Need to debug why gaugesoln.number always 1 here
-    pylab.title("%s at Gauge %i" % (plotitem._plotaxes.title,\
-                 gaugesoln.number))
-
-    pylab.xlabel("time")
 
     if (plot_type in ['1d_plot']) and (plotstyle != ''):
         if color:
             kwargs['color'] = color
 
         pobj = pylab.plot(t,var,plotstyle,**kwargs)
-        # plotcommand = "pobj=pylab.plot(t,var,'%s', **kwargs)"  \
-        #               % plotstyle
-        # exec(plotcommand)
 
 
     elif plot_type == '1d_empty':
@@ -433,21 +435,21 @@ def plotgauge1(gaugesoln, plotitem, current_data):
 
     return current_data
     
-def read_setgauges(datadir):
+def read_setgauges(outdir):
     """
-    Read the info from setgauges.data.
+    Read the info from gauges.data.
     """
 
     try:
         import clawpack.amrclaw.data as amrclaw
     except ImportError as e:
-        print "You must have AMRClaw installed to plot gauges."
-        print "continuing..."
+        print("You must have AMRClaw installed to plot gauges.")
+        print("continuing...")
         return None
 
     setgauges = amrclaw.GaugeData()
     try:
-        setgauges.read(datadir)
+        setgauges.read(outdir)
     except IOError as e:
         # No gauges.data file was found, ignore this exception
         pass
@@ -462,22 +464,22 @@ def plot_gauge_locations(plotdata, gaugenos='all', mapc2p=None, \
     Plot gauge locations on current axes.
     format_string specifies the symbol to be plotted.
     If add_labels==True then labels will also be added (gauge number).
-    This routine determines locations from the file setgauges.data in
+    This routine determines locations from the file gauges.data in
     directory plotdata.rundir.  It does not require reading in the fort.gauge file
     produced by running the code.
     """
 
     from pylab import figure, plot, clf, title, text
 
-    datadir = plotdata.rundir  # this should contain setgauges.data
+    datadir = plotdata.rundir  # this should contain gauges.data
 
     try:
-        setgauges = read_setgauges(datadir)
+        setgauges = read_setgauges(plotdata.outdir)
     except:
         return
 
     if len(setgauges.gauges) == 0:
-        print "*** plot_gauge_locations: No gauges specified in setgauges.data"
+        print("*** plot_gauge_locations: No gauges specified in gauges.data")
         return
 
     if gaugenos=='all':
@@ -501,7 +503,7 @@ def plot_gauge_locations(plotdata, gaugenos='all', mapc2p=None, \
                 yn = yn + yoffset
                 text(xn,yn,'  %s' % gauge[0], fontsize=fontsize)
         except:
-            print "*** plot_gauge_locations: warning: did not find x,y data for gauge ",gauge[0]
+            print("*** plot_gauge_locations: warning: did not find x,y data for gauge ",gauge[0])
 
 
 #------------------------------------------------------------------------
@@ -532,7 +534,7 @@ def printfig(fname='',gaugeno='', figno='', format='png', plotdir='.', \
     pylab.figure(figno)
     if plotdir != '.':
        fname = os.path.join(plotdir,fname)
-    if verbose:  print '    Saving plot to file ', fname
+    if verbose:  print('    Saving plot to file ', fname)
     pylab.savefig(fname)
 
 
@@ -563,12 +565,12 @@ def printgauges(plotdata=None, verbose=True):
     from clawpack.visclaw import plotpages
 
 
-    if not sys.modules.has_key('matplotlib'):
-        print '*** Error: matplotlib not found, no plots will be done'
+    if 'matplotlib' not in sys.modules:
+        print('*** Error: matplotlib not found, no plots will be done')
         return plotdata
         
     if not isinstance(plotdata,ClawPlotData):
-        print '*** Error, plotdata must be an object of type ClawPlotData'
+        print('*** Error, plotdata must be an object of type ClawPlotData')
         return plotdata
 
     plotdata._mode = 'printframes'
@@ -591,8 +593,8 @@ def printgauges(plotdata=None, verbose=True):
         overwrite = plotdata.overwrite # ok to overwrite?
         msgfile = plotdata.msgfile     # where to write error messages
     except:
-        print '*** Error in printframes: plotdata missing attribute'
-        print '  *** plotdata = ',plotdata
+        print('*** Error in printframes: plotdata missing attribute')
+        print('  *** plotdata = ',plotdata)
         return plotdata
 
     if fignos == 'all':
@@ -627,21 +629,21 @@ def printgauges(plotdata=None, verbose=True):
     rootdir = os.getcwd()
 
     # annoying fix needed when EPD is used for plotting under cygwin:
-    if rootdir[0:9] == 'C:\cygwin' and outdir[0:9] != 'C:\cygwin':
-        outdir = 'C:\cygwin' + outdir
+    if rootdir[0:9] == r'C:\cygwin' and outdir[0:9] != r'C:\cygwin':
+        outdir = r'C:\cygwin' + outdir
         plotdata.outdir = outdir
-    if rootdir[0:9] == 'C:\cygwin' and rundir[0:9] != 'C:\cygwin':
-        rundir = 'C:\cygwin' + rundir
+    if rootdir[0:9] == r'C:\cygwin' and rundir[0:9] != r'C:\cygwin':
+        rundir = r'C:\cygwin' + rundir
         plotdata.rundir = rundir
-    if rootdir[0:9] == 'C:\cygwin' and plotdir[0:9] != 'C:\cygwin':
-        plotdir = 'C:\cygwin' + plotdir
+    if rootdir[0:9] == r'C:\cygwin' and plotdir[0:9] != r'C:\cygwin':
+        plotdir = r'C:\cygwin' + plotdir
         plotdata.plotdir = plotdir
 
     try:
         os.chdir(rundir)
     except:
-        print '*** Error: cannot move to run directory ',rundir
-        print 'rootdir = ',rootdir
+        print('*** Error: cannot move to run directory ',rundir)
+        print('rootdir = ',rootdir)
         return plotdata
 
 
@@ -653,7 +655,7 @@ def printgauges(plotdata=None, verbose=True):
     try:
         plotpages.cd_plotdir(plotdata)
     except:
-        print "*** Error, aborting plotframes"
+        print("*** Error, aborting plotframes")
         return plotdata
 
 
@@ -665,8 +667,8 @@ def printgauges(plotdata=None, verbose=True):
             os.remove(file)
     else:
         if len(framefiles) > 1:
-            print "*** Remove frame*.png and frame*.html and try again,"
-            print "  or use overwrite=True in call to printframes"
+            print("*** Remove frame*.png and frame*.html and try again,")
+            print("  or use overwrite=True in call to printframes")
             return plotdata
 
     
@@ -676,7 +678,7 @@ def printgauges(plotdata=None, verbose=True):
     try:
         os.chdir(outdir)
     except:
-        print '*** Error printframes: cannot move to outdir = ',outdir
+        print('*** Error printframes: cannot move to outdir = ',outdir)
         return plotdata
 
 
@@ -692,7 +694,7 @@ def printgauges(plotdata=None, verbose=True):
             pngfile[frameno,figno] = 'frame' + file[-4:] + 'fig%s.png' % figno
     
     if len(fortfile) == 0:
-        print '*** No fort.q files found in directory ', os.getcwd()
+        print('*** No fort.q files found in directory ', os.getcwd())
         return plotdata
     
     # Discard frames that are not from latest run, based on
@@ -701,9 +703,9 @@ def printgauges(plotdata=None, verbose=True):
 
     numframes = len(framenos)
 
-    print "Will plot %i frames numbered:" % numframes, framenos
-    print 'Will make %i figure(s) for each frame, numbered: ' \
-          % len(fignos_each_frame), fignos_each_frame
+    print("Will plot %i frames numbered:" % numframes, framenos)
+    print('Will make %i figure(s) for each frame, numbered: ' \
+          % len(fignos_each_frame), fignos_each_frame)
 
     #fignames = {}
     #for figname in plotdata._fignames:
@@ -736,15 +738,15 @@ def printgauges(plotdata=None, verbose=True):
     # -----------------------------------------
 
     if not plotdata.printfigs:
-        print "Using previously printed figure files"
+        print("Using previously printed figure files")
     else:
-        print "Now making png files for all figures..."
+        print("Now making png files for all figures...")
         for frameno in framenos:
             plotframe(frameno, plotdata, verbose)
-            print 'Frame %i at time t = %s' % (frameno, frametimes[frameno])
+            print('Frame %i at time t = %s' % (frameno, frametimes[frameno]))
         for gaugeno in gaugenos:
             plotgauge(gaugeno, plotdata, verbose)
-            print 'Gauge %i ' % gaugeno
+            print('Gauge %i ' % gaugeno)
 
 
     if plotdata.latex:
@@ -755,14 +757,14 @@ def printgauges(plotdata=None, verbose=True):
     #-------
     
     if plotdata.gif_movie:
-        print 'Making gif movies.  This may take some time....'
+        print('Making gif movies.  This may take some time....')
         for figno in fignos_each_frame:
             try:
                 os.system('convert -delay 20 frame*fig%s.png moviefig%s.gif' \
                    % (figno,figno))
-                print '    Created moviefig%s.gif' % figno
+                print('    Created moviefig%s.gif' % figno)
             except:
-                print '*** Error creating moviefig%s.gif' % figno
+                print('*** Error creating moviefig%s.gif' % figno)
     
     os.chdir(rootdir)
 
@@ -777,3 +779,84 @@ def printgauges(plotdata=None, verbose=True):
     return plotdata
     # end of printframes
 
+
+def compare_gauges(outdir1, outdir2, gaugenos='all', q_components='all',
+                    tol=0., verbose=True, plot=False):
+
+    """
+    Compare gauge output in two output directories.
+
+    :Input:
+     - *outdir1, outdir2* -- output directories
+     - *gaugenos* -- list of gauge numbers to compare, or 'all' in which case
+       outdir1/gauges.data will be used to determine gauge numbers.
+     - *q_components* -- list of components of q to compare.
+     - *tol* -- tolerance for checking equality
+     - *verbose* -- print out dt and dq for each comparison?
+     - *plot* -- if True, will produce a plot for each gauge, with a
+       subfigure for each component of q.
+
+
+    Returns True if everything matches to tolerance *tol*, else False.
+    """
+
+    from clawpack.visclaw.data import ClawPlotData
+    from matplotlib import pyplot as plt
+    
+    if gaugenos == 'all':
+        # attempt to read from gauges.data:
+        try:
+            setgauges1 = read_setgauges(outdir1)
+            setgauges2 = read_setgauges(outdir2)
+        except:
+            print('*** could not read gauges.data from one of the outdirs')
+            return
+        gaugenos = setgauges1.gauge_numbers
+        if setgauges2.gauge_numbers != gaugenos:
+            print('*** warning -- outdirs have different sets of gauges')
+
+        if len(gaugenos)==0:
+            print("*** No gauges found in gauges.data")
+            return
+
+    plotdata1 = ClawPlotData()
+    plotdata1.outdir = outdir1
+    plotdata2 = ClawPlotData()
+    plotdata2.outdir = outdir2
+
+    matches = True
+    for gaugeno in gaugenos:
+        g1 = plotdata1.getgauge(gaugeno,verbose=verbose)
+        t1 = g1.t
+        q1 = g1.q
+
+        g2 = plotdata2.getgauge(gaugeno,verbose=verbose)
+        t2 = g2.t
+        q2 = g2.q
+
+        dt = abs(t1-t2).max()
+        if verbose:
+            print("Max difference in t[:] at gauge %s is %g" % (gaugeno,dt))
+        matches = matches and (dt <= tol)
+
+        if q_components == 'all':
+            q_components = list(range(q1.shape[0]))
+
+        for m in q_components:
+            dq = abs(q1[m,:]-q2[m,:]).max()
+            if verbose:
+                print("Max difference in q[%s] at gauge %s is %g" % (m,gaugeno,dq))
+            matches = matches and (dq <= tol)
+
+        if plot:
+            plt.figure(gaugeno)
+            plt.clf()
+            mq = len(q_components)
+            for k,m in enumerate(q_components):
+                plt.subplot(mq,1,k+1)
+                plt.plot(g1.t,g1.q[m,:],'b',label='outdir1')
+                plt.plot(g2.t,g2.q[m,:],'r',label='outdir2')
+                plt.legend()
+                plt.title('q[%s] at gauge number %s' % (m,gaugeno))
+        
+    return matches       
